#!/bin/ksh

# Script to take an LDIF definition of one or more groups and other
# ldif data and add the LDIF data to make any groups into 
# AM groups.  Any non-group LDIF is passed along the output
# stream unchanged.

# This script may need to be customized for the following item:
# - Where does the AM group name come from?  AM stores the group
#   name in the "cn:" attribute of the SecGroup object.  A
#   common choice is to name the AM group object using the
#   "cn:" attribute of the LDAP group object, which is the
#   approach taken by this script.

# Input:
# 	Domain name argument
# 	Standard input device for LDIF data
# Output: standard output device

# Restrictions:
# This script can be run under the context of any user.  It does not
# require write authority to the current directory.

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

usage(){
	print "Usage: $0 <Access Manager domain name> [new|old] [acls]"
	print "  where"
	print "    <Access Manager domain name> is either \"Default\" if"
	print "      users are in the Default domain or the domain name"
	print "      if users are in a Access Manager sub-domain."
	print "    old|new is is an optional keyword that is either \"old\" for"
	print "      the old TAM LDAP data format or \"new\" for the new TAM LDAP"
	print "      data format.  If not specified, it defaults to \"new\"."
	print "    \"acls\" is an optional keyword that, when used, generates"
	print "      Access Manager ACL definitions on secUser objects."
	print ""
	print "    Note that there are no ACLs on user objects in the \"new\""
	print "    data format, no ACLs appear in the output when the \"new\","
	print "    data format and the \"acl\" options are both specified."
	exit 1
}

if [ X$1 = X ];then
	usage
fi
domain=$1

if [ X$2 = Xnew ];then
	dataform=new
elif [ X$2 = Xold ];then
	dataform=old
else
	dataform=new
fi

acls=0
if [ X$2 != X ]&&[ X$3 = Xacls ];then
	acls=1
fi

# Figure out which uuidgen program to use

if [[ "X`which uuidgen | $AWK '{print $2}'`" != "X" ]] && \
    [[ "X`which /opt/PolicyDirector/sbin/pduuidgen | $AWK '{print $2}'`" != "X" ]] then
	print "$0 requires pduuidgen from the Access Manager product"
	print "    or uuidgen from some other product."
	print "    It is sufficient to copy the pduuidgen executable"
	print "    and any associated libraries to meet this requirement."
	exit 1
fi
if [ -f /opt/PolicyDirector/sbin/pduuidgen ];then
	uuidgenp=/opt/PolicyDirector/sbin/pduuidgen
else
	uuidgenp=uuidgen

	# Determine whether the uuidgen program supports the -n option
	if [ `uuidgen -? 2>&1 | grep "\-n" | wc -l` = 0 ];then
		print "$0 - ERROR:"
		print "    The uuidgen program that was found does not support the -n option."
		print "    There are performance reasons for not using the uuidgen program"
		print "    without this option.  It is recommended that the Tivoli Access"
		print "    Manager RTE be installed so tha the pduuidgen utility from that"
		print "    product will be used."
		exit 1
	fi
fi

# Have the uuidgen program create 1M uuids at a time
uuidgenp="$uuidgenp -n 1000000"

$AWK -v uuidgenp="$uuidgenp" \
    -v domain=$domain \
    -v dataform=$dataform \
    -v acls=$acls '

function print_pd_def(){

	# If this is a group, create the AM group LDIF
	if (is_group){

		# dump out AM group ldif definition
		# first, generate the uuid
		if (uuidgenp | getline uuid <= 0){
			# we ran out of uuids, so start
			# another uuidgen
			close(uuidgenp)
			uuidgenp | getline uuid
		}

		# Create the AM group LDIF

		if (dataform == "new"){

			if (tolower(domain) == "default"){
				print "dn: cn="groupname",cn=Groups,secAuthority=Default"
			} else {
				print "dn: cn="groupname",cn=Groups,secAuthority="domain",cn=Subdomains,secAuthority=Default"
			}
			print "objectclass: secGroup"
			print "objectclass: top"
			print "secAuthority: "domain
			print "cn: "groupname
			print "secUUID: "uuid
			print "secHasPolicy: false"
			print "secDN: "save_dn
			print ""

		}else{ # old data format

			print "dn: secAuthority="domain","save_dn
			print "objectclass: secGroup"
			print "objectclass: top"
			print "secAuthority: "domain
			print "cn: "groupname
			print "secUUID: "uuid
			print "secHasPolicy: false"
			if (acls){
			    print "aclpropagate: TRUE"
  			    if (tolower(domain) == "default"){
				print "aclentry: group:CN=REMOTE-ACL-USERS,CN=SECURITYGROUPS,SECAUTHORITY=DEFAULT:nor"
				print " mal:rsc:at.secAcctValid:rwsc:at.secPwdFailCountTime:rwsc:at.secPwdFailures:r"
				print " wsc:at.secPwdLastChanged:rwsc:at.secPwdLastFailed:rwsc:at.secPwdLastUsed:rws"
				print " c:at.secPwdUnlockTime:rwsc:at.secPwdValid:rwsc"
				print "aclentry: group:CN=IVACLD-SERVERS,CN=SECURITYGROUPS,SECAUTHORITY=DEFAULT:norma"
				print " l:rsc:at.secAcctValid:rwsc:at.secPwdFailCountTime:rwsc:at.secPwdFailures:rws"
				print " c:at.secPwdLastChanged:rwsc:at.secPwdLastFailed:rwsc:at.secPwdLastUsed:rwsc:"
				print " at.secPwdUnlockTime:rwsc:at.secPwdValid:rwsc"
				print "aclentry: group:CN=SECURITYGROUP,SECAUTHORITY=DEFAULT:object:ad:normal:rwsc:se"
				print " nsitive:rwsc:critical:rwsc"
		    	    } else {
				print "aclentry: group:CN=SECURITYGROUP,SECAUTHORITY=DEFAULT:object:ad:normal:rwsc:se"
				print " nsitive:rwsc:critical:rwsc"
				print "aclentry: group:CN=REMOTE-ACL-USERS,CN=SECURITYGROUPS,SECAUTHORITY="domain",CN=SUB"
				print " DOMAINS,SECAUTHORITY=DEFAULT:normal:rsc:at.secAcctValid:rwsc:at.secPwdFailCo"
				print " untTime:rwsc:at.secPwdFailures:rwsc:at.secPwdLastChanged:rwsc:at.secPwdLastF"
				print " ailed:rwsc:at.secPwdLastUsed:rwsc:at.secPwdUnlockTime:rwsc:at.secPwdValid:rw"
				print " sc"
				print "aclentry: group:CN=IVACLD-SERVERS,CN=SECURITYGROUPS,SECAUTHORITY="domain",CN=SUBDO"
				print " MAINS,SECAUTHORITY=DEFAULT:normal:rsc:at.secAcctValid:rwsc:at.secPwdFailCoun"
				print " tTime:rwsc:at.secPwdFailures:rwsc:at.secPwdLastChanged:rwsc:at.secPwdLastFai"
				print " led:rwsc:at.secPwdLastUsed:rwsc:at.secPwdUnlockTime:rwsc:at.secPwdValid:rwsc"
				print "aclentry: group:CN=SECURITYGROUP,SECAUTHORITY="domain",CN=SUBDOMAINS,SECAUTHORITY="
				print " DEFAULT:object:ad:normal:rwsc:sensitive:rwsc:critical:rwsc"
		    	    }
			}
			if (tolower(domain) == "default"){
				print ""
				print "dn: secUUID="uuid",cn=Groups,secAuthority=Default"
			} else {
				print ""
				print "dn: secUUID="uuid",cn=Groups,secAuthority="domain",cn=Subdomains,secAuthority=Default"
			}
			print "objectclass: secMap"
			print "objectclass: top"
			print "secDN: "save_dn
			print "secUUID: "uuid
			print ""
		}
	}
}

function getprintattr(){

	attr_tmp = substr($0, index($0,$2))
	print $0
	getline
	while (substr($0,1,1)==" "){
		attr_tmp = attr_tmp""substr($0,index($0,$1))
		print $0
		getline
	}
	return attr_tmp
}

# main()
{

	if ($1 == "dn:"){

		if (notfirstdn){
			print_pd_def()
		}
		notfirstdn = 1

		save_dn = getprintattr()
	}

	if ($1 == "cn:"){

		# note: using cn field for groupname
		groupname = getprintattr()
	}

	# Another option for group test: ($1 == "objectclass:" && $2 == "accessGroup")
	if ($1 == "member:") {

		is_group = 1
	}

	# print non-matched ldif line
	print $0

} END {

	if (notfirstdn){
		print ""
		print_pd_def()
	}
}'


