#!/bin/ksh

# Script to take an LDIF definition of many users and output the LDIF
# for that original definition plus the LDIF to make that user a AM
# user.

# This script must be customized based upon the input user LDIF
# Some of the questions to be answered are
# - Where does the AM principalname come from?
#   In this script, the principal name comes from the "uid"
#   attribute.  Another common choice is the "cn" attribute.
# - What settings should be used for the following attributes?
#	secpwdvalid (true as in below)
#	sechaspolicy (false as in below)
#	secpwdlastchanged (see setting below)
#	secacctvalid (true as in below)

# Input: 
# 	Domain name argument
# 	Standard input device for LDIF data
# Output: standard output device

# Restrictions:
# This script can be run under the context of any user.  It does not
# require write authority to the current directory.

# set the password last change value here
# It is a good idea to set this in the past and force a password change.
# In this case, we are setting it to the future, so passwords are initially valid.
secpwdlastchanged=20151208214816.0Z

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

usage(){
	print "Usage: $0 <Access Manager domain name> [new|old] [acls]"
	print "  where"
	print "    <Access Manager domain name> is either \"Default\" if"
	print "      users are in the Default domain or the domain name"
	print "      if users are in a Access Manager sub-domain."
	print "    old|new is is an optional keyword that is either \"old\" for"
	print "      the old TAM LDAP data format or \"new\" for the new TAM LDAP"
	print "      data format.  If not specified, it defaults to \"new\"."
	print "    \"acls\" is an optional keyword that, when used, generates"
	print "      Access Manager ACL definitions on secUser objects."
	print ""
	print "    Note that there are no ACLs on user objects in the "\new\""
	print "    data format, no ACLs appear in the output when the "\new\","
	print "    data format and the \"acl\" options are both specified."
	exit 1
}

if [ X$1 = X ];then
	usage
fi
domain=$1

if [ X$2 = Xnew ];then
	dataform=new
elif [ X$2 = Xold ];then
	dataform=old
else
	dataform=new
fi

acls=0
if [ X$2 != X ]&&[ X$3 = Xacls ];then
	acls=1
fi

# Figure out which uuidgen program to use

if [[ "X`which uuidgen | $AWK '{print $2}'`" != "X" ]] && \
    [[ "X`which /opt/PolicyDirector/sbin/pduuidgen | $AWK '{print $2}'`" != "X" ]] then
	print "$0 requires pduuidgen from the Access Manager product"
	print "    or uuidgen from some other product."
	print "    It is sufficient to copy the pduuidgen executable"
	print "    and any associated libraries to meet this requirement."
	exit 1
fi
if [ -f /opt/PolicyDirector/sbin/pduuidgen ];then
	uuidgenp=/opt/PolicyDirector/sbin/pduuidgen
else
	uuidgenp=uuidgen

	# Determine whether the uuidgen program supports the -n option
	if [ `uuidgen -? 2>&1 | grep "\-n" | wc -l` = 0 ];then
		print "$0 - ERROR:"
		print "    The uuidgen program that was found does not support the -n option."
		print "    There are performance reasons for not using the uuidgen program"
		print "    without this option.  It is recommended that the Tivoli Access"
		print "    Manager RTE be installed so tha the pduuidgen utility from that"
		print "    product will be used."
		exit 1
	fi
fi

$AWK -v uuidgenp="$uuidgenp" \
    -v secpwdlastchanged=$secpwdlastchanged \
    -v domain=$domain \
    -v dataform=$dataform \
    -v acls=$acls '

function print_pd_def(){

    if (principalname != ""){

	# dump out AM user ldif definition
	# first, generate the uuid
	uuidgenpn | getline uuid
	if (++num_uuidgenps >= pipe_size){
		# we ran out of uuids, so close the pipe
		close(uuidgenpn)
		num_uuidgenps = 0
	}
	#print uuid

	# print the AM definition ldif

	if (dataform == "new"){

		if (tolower(domain) == "default"){
			print "dn: principalName="principalname",cn=Users,secAuthority=Default"
		} else {
			print "dn: principalName="principalname",cn=Users,secAuthority="domain",cn=Subdomains,secAuthority=Default"
		}
		print "objectclass: secUser"
		print "objectclass: eUser"
		print "objectclass: cimManagedElement"
		print "objectclass: top"
		print "secAuthority: "domain
		print "secLoginType: Default:LDAP"
		print "secAcctValid: true"
		print "principalName: "principalname
		print "secUUID: "uuid
		print "secHasPolicy: false"
		print "secDomainId: "domain"%"principalname
		print "secDN: "userdn
		print "secPwdLastChanged: "secpwdlastchanged
		print "secPwdValid: true"
		print ""

	}else{ # old data format

		print "dn: secAuthority="domain","userdn
		print "objectclass: secUser"
		print "objectclass: eUser"
		print "objectclass: cimManagedElement"
		print "objectclass: top"
		print "secAuthority: "domain
		print "secLoginType: Default:LDAP"
		print "secPwdValid: true"
		print "principalName: "principalname
		print "secUUID: "uuid
		print "secHasPolicy: false"
		print "secPwdLastChanged: "secpwdlastchanged
		print "secAcctValid: true"
		print "secDomainId: "domain"%"principalname
		if (acls){
		    print "aclpropagate: TRUE"
		    if (tolower(domain) == "default"){
			print "aclentry: group:CN=REMOTE-ACL-USERS,CN=SECURITYGROUPS,SECAUTHORITY=DEFAULT:nor"
			print " mal:rsc:at.secAcctValid:rwsc:at.secPwdFailCountTime:rwsc:at.secPwdFailures:r"
			print " wsc:at.secPwdLastChanged:rwsc:at.secPwdLastFailed:rwsc:at.secPwdLastUsed:rws"
			print " c:at.secPwdUnlockTime:rwsc:at.secPwdValid:rwsc"
			print "aclentry: group:CN=IVACLD-SERVERS,CN=SECURITYGROUPS,SECAUTHORITY=DEFAULT:norma"
			print " l:rsc:at.secAcctValid:rwsc:at.secPwdFailCountTime:rwsc:at.secPwdFailures:rws"
			print " c:at.secPwdLastChanged:rwsc:at.secPwdLastFailed:rwsc:at.secPwdLastUsed:rwsc:"
			print " at.secPwdUnlockTime:rwsc:at.secPwdValid:rwsc"
			print "aclentry: group:CN=SECURITYGROUP,SECAUTHORITY=DEFAULT:object:ad:normal:rwsc:se"
			print " nsitive:rwsc:critical:rwsc"
		    } else {
			print "aclentry: group:CN=SECURITYGROUP,SECAUTHORITY=DEFAULT:object:ad:normal:rwsc:se"
			print " nsitive:rwsc:critical:rwsc"
			print "aclentry: group:CN=REMOTE-ACL-USERS,CN=SECURITYGROUPS,SECAUTHORITY="domain",CN=SUB"
			print " DOMAINS,SECAUTHORITY=DEFAULT:normal:rsc:at.secAcctValid:rwsc:at.secPwdFailCo"
			print " untTime:rwsc:at.secPwdFailures:rwsc:at.secPwdLastChanged:rwsc:at.secPwdLastF"
			print " ailed:rwsc:at.secPwdLastUsed:rwsc:at.secPwdUnlockTime:rwsc:at.secPwdValid:rw"
			print " sc"
			print "aclentry: group:CN=IVACLD-SERVERS,CN=SECURITYGROUPS,SECAUTHORITY="domain",CN=SUBDO"
			print " MAINS,SECAUTHORITY=DEFAULT:normal:rsc:at.secAcctValid:rwsc:at.secPwdFailCoun"
			print " tTime:rwsc:at.secPwdFailures:rwsc:at.secPwdLastChanged:rwsc:at.secPwdLastFai"
			print " led:rwsc:at.secPwdLastUsed:rwsc:at.secPwdUnlockTime:rwsc:at.secPwdValid:rwsc"
			print "aclentry: group:CN=SECURITYGROUP,SECAUTHORITY="domain",CN=SUBDOMAINS,SECAUTHORITY="
			print " DEFAULT:object:ad:normal:rwsc:sensitive:rwsc:critical:rwsc"
		    }
		}
		if (tolower(domain) == "default"){
			print ""
			print "dn: secUUID="uuid",cn=Users,secAuthority=Default"
		} else {
			print ""
			print "dn: secUUID="uuid",cn=Users,secAuthority="domain",cn=Subdomains,secAuthority=Default"
		}
		print "objectclass: secMap"
		print "objectclass: top"
		print "secDN: "userdn
		print "secUUID: "uuid
		print ""
		print "dn: cn=PolicyData,secAuthority="domain","userdn
		print "objectclass: secPolicyData"
		print "objectclass: top"
		print "cn: PolicyData"
		print "secPwdLastChanged: "secpwdlastchanged
		print ""
	}
	principalname = ""
    }
}

function getprintattr(){

	attr_tmp = substr($0, index($0,$2))
	print $0
	getline
	while (substr($0,1,1)==" "){
		attr_tmp = attr_tmp""substr($0,index($0,$1))
		print $0
		getline
	}
	return attr_tmp
}

BEGIN{

	pipe_size = 10000
	uuidgenpn = uuidgenp" -n "pipe_size

}

# main()
{

	if ($1 == "dn:"){

		if (notfirstdn){
			print_pd_def()
		}
		notfirstdn = 1

		userdn = getprintattr()
	}

	if ($1 == "uid:"){

		# note: using uid field for
		# AM principalname
		principalname = getprintattr()
	}

	# print non-dn, non-uid ldif line
	print $0

} END {

	if (notfirstdn){
		print ""
		print_pd_def()
	}

	# hack: clean the pipe to prevent "Broken Pipe" errors
	while (++num_uuidgenps < pipe_size){
		uuidgenpn | getline uuid
	}

}'


