#!/bin/ksh

# Script to add IDS LDAP ACLs for TAM user objects to an LDIF input
# stream consisting of a db2ldif dump of a directory that had
# previously been "fixed up" using the TAM tuning guide fixacls script.
#
# Since only the old TAM LDAP format needed ACLs to be fixed up,
# this script only works on users in the old TAM LDAP format.
#
# The resulting LDIF output stream contains all of the LDIF of
# the input stream with TAM ACLS added to TAM user objects that
# don't already have ACL defined to them.  If a TAM user object
# already has an ACL, it is assumed to be correct and thus is
# not modified.

# Note that a requirement of this script is that all domain
# objects precede the definition of user objects for their respective
# domains.  Part of this is dependent upon the behavior of the
# LDAP utilities.  Currently, IDS db2ldif dumps objects in the
# order they were created.  The TAM tuning guide scripts guide
# you through creating the minimum unconfigured TAM configuration
# before creating the users.  So, this requirement is met with
# IDS.

# Input: standard input device containing ldif data
# Output: standard output device with augmented ldif data

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

$AWK -v opt=$1 'BEGIN{
	objectclass = toupper("objectclass")
	secAuthorityInfo = toupper("secAuthorityInfo")
	secUser = toupper("secUser")
	secauthority = toupper("secauthority")
	aclentry = toupper("aclentry")
}{
	# process a line of ldif at a time
	w1 = toupper($1)
	w2 = toupper($2)
	if (w1 == objectclass":" && w2 == secAuthorityInfo){
		# this is a TAM domain object
		is_tam_domain = 1

	}else if (w1 == objectclass":" && w2 == secUser){
		# found a TAM user object
		is_tam_user = 1

	}else if (w1 == secauthority":"){
		#  this object has a secauthority attribute
		secauth = $2

	}else if (w1 == aclentry":"){
		# this object contains an ACL
		is_acl_entry = 1

		# print "DEBUG: is_tam_domain: "is_tam_domain" , secauth: "secauth

		# if this is an ACL within a TAM domain definition,
		# save the ACL.
		if (is_tam_domain){
			not_end_of_acl = 1
			while (not_end_of_acl){
				if ( tam_domain_acls[secauth] == "" ){
					tam_domain_acls[secauth] = $0
				}else{
					tam_domain_acls[secauth] = tam_domain_acls[secauth]"\n"$0
				}
				print $0
				if (getline == 0){
					# hit the end of the input file
					exit
				}
				w1 = toupper($1)
				if (w1 != aclentry":" && (substr($0,1,1) != " " || substr($0,2,1) == "")){
					not_end_of_acl = 0
				}
			}
		}

		# print "DEBUG: "tam_domain_acls[secauth]
	}

	if ($1 == ""){
		# end of ldap object definition

		# print "DEBUG: is_tam_user: "is_tam_user" , is_acl_entry: "is_acl_entry" , secauth: "secauth

		if (is_tam_user && !is_acl_entry){
			# the previous object was a TAM user and
			# it did not contain an ACL

			# output the appropriate ACLs for the TAM user
	    		print "aclpropagate: TRUE"
			print tam_domain_acls[secauth]
		}

		is_tam_domain = 0
		secauth = ""
		is_tam_user = 0
		is_acl_entry = 0
	}

	# finally, echo it to the output stream
	print $0

}'


