#!/bin/ksh

# script to check and generate ldif for applying TAM ACLs to  LDAP suffixes.  These
# ACLs are typically set during TAM configuration, but may need to be set before
# that time if a bulkload will be performed before TAM configuration.  If the TAM
# ACLs are not applied until after bulkload, the TAM configuration will be slow
# due to the propagation of ACLs.

# Defaults: - LDAP root administrator is cn=root, if not specified
#           - LDAP port is 389, if not specified

# Restrictions:
# This script can be run under the context of any user.  It
# requires write authority to the current directory for temporary
# files.

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

usage(){

print "Usage: $0 <ldap host> <ldap password> [ <ldap admin> <ldap port> ]"
print "     For example:  $0 localhost adminpass cn=root 389 " 
exit 1

}

if [ "X$2" = "X" ];then
        usage
fi

ldaphost=$1
ldappwd=$2

if [ "X$3" = "X" ];then
        ldapadmin=cn=root
else
	ldapadmin=$3
fi
if [ "X$4" = "X" ];then
        ldapport=389
else
	ldapport=$4
fi

ldapcred="-h $ldaphost -p $ldapport -D $ldapadmin -w $ldappwd"

# A function to compare the results of the suffix searches on aclentry and entryowner attributes and
# generate ldif to add anything that is missing.
#
# This function is designed for the aclentry and entryowner attributes

search_compare_fix_attrs(){

	# input
	srchbase=$1
	desired_attrs_fn=$2 # a file name
	attr=$3

	# Get the current entry attribute definitions
	ldapsearch $ldapcred -s base -b "$srchbase" "objectclass=*" $attr 2>cur_errors.tmp |
		tail +2 | sort >cur_attrs.tmp

	if [ "X`cat cur_attrs.tmp`" = "X" ] || [ "X`cat cur_errors.tmp`" != "X" ];then
		rm -f cur_errors.tmp
		rm -f cur_attrs.tmp
		return
	fi
	rm -f cur_errors.tmp

	# Add the current entry attribute definitions to the desired ones
	cat cur_attrs.tmp $desired_attrs_fn | sort | uniq >desired_attrs.tmp2

	# Get just the attribute values for current and desired
	cat cur_attrs.tmp | $AWK -F"=" '{print substr($0,index($0,$2))}' | sort >cur_attr_values.tmp
	rm -f cur_attrs.tmp
	cat desired_attrs.tmp2 | $AWK -F"=" '{print substr($0,index($0,$2))}' | sort >desired_attr_values.tmp

	# if the current LDAP entry is not the source of the attribute value, in this case
	# its not aclsource or ownersource, then clear out the list of current entry attribute
	# values.  The attribute values did not come from this entry.  They were inherited.

	srchbase_cap=`echo $srchbase | $AWK '{print toupper($0)}'`
	if [ $attr = "aclentry" ];then

		aclsrc=`ldapsearch $ldapcred -s base -b "$srchbase" "objectclass=*" aclsource |
			grep -i aclsource |
			$AWK -F"=" '{print toupper(substr($0,index($0,$2)))}'`

		# if the aclsource is not this entry, clear out the list current attributes
		if [ "$srchbase_cap" != "$aclsrc" ];then

			echo "" >cur_attr_values.tmp
		fi
	fi
	if [ $attr = "entryowner" ];then

		ownsrc=`ldapsearch $ldapcred -s base -b "$srchbase" "objectclass=*" ownersource |
			grep -i ownersource |
			$AWK -F"=" '{print toupper(substr($0,index($0,$2)))}'`

		# if the ownersource is not this entry, clear out the list current attributes
		if [ "$srchbase_cap" != "$ownsrc" ];then

			echo "" >cur_attr_values.tmp
		fi
	fi

	# split, sort then re-join the types within the aclentry lines
	if [ $attr = "aclentry" ];then

		sort_acls(){
			$AWK '{
				n=split($0,a,":")
				if (n < 4){
					print $0
				} else {
					system("rm -f attr_reord.tmp")
					printf("%s",a[1]":"a[2]":")
					for(i=3;i<=n;i+=2){
						print a[i]":"a[i+1] >"attr_reord.tmp"
					}
					close("attr_reord.tmp")
					system("cat attr_reord.tmp | sort >attr_reord.tmp2")
					system("mv attr_reord.tmp2 attr_reord.tmp")

					# rejoin the line
					x=""
					while (getline < "attr_reord.tmp"){
						x=x""$0
					}
					close("attr_reord.tmp")
					system("rm -f attr_reord.tmp")
					print x
				}
			}'
		}

		cat desired_attr_values.tmp | sort_acls | uniq >desired_attr_values.tmp2
		mv desired_attr_values.tmp2 desired_attr_values.tmp

		cat cur_attr_values.tmp | sort_acls >cur_attr_values.tmp2
		mv cur_attr_values.tmp2 cur_attr_values.tmp

		eliminate_dup_ids(){
			$AWK '{
				n=split($0,a,":")
				id = a[1]":"a[2]
				if (id != prev_id){
					if (prev_attr != ""){
						print prev_attr
					}
					prev_id = id
					prev_attr = $0
				}else{
					# Pick the duplicate ID of the longest length
					# kind of arbitrary, but the TAM acls do not seem consistent
					if (length($0) > length(prev_attr)){
						prev_id = id
						prev_attr = $0
					}
				}
				
			}END{
				print prev_attr
			}'
		}

		cat desired_attr_values.tmp | eliminate_dup_ids | uniq >desired_attr_values.tmp2
		mv desired_attr_values.tmp2 desired_attr_values.tmp
	fi

	# determine what needs to change and generate the LDIF to do it
	diff desired_attr_values.tmp cur_attr_values.tmp | grep "<" | \
			$AWK -v dn="$srchbase" -v desired_attrs_fn=desired_attrs.tmp2 '{

		# Find the attribute in the desired_attrs_fn file
		split(substr($0,index($0,$2)),a,":")
		search_attr = a[1]":"a[2]
		while (getline < desired_attrs_fn){
			split(substr($0,index($0,"=")+1),a,":")
			if (toupper(a[1]":"a[2]) == toupper(search_attr)){
				break
			}
		}
		close(desired_attrs_fn)

		attr = substr($0,1,index($0,"=")-1)
		val = substr($0,index($0,"=")+1)

		if (!subsequent_to_first_time){
			subsequent_to_first_time = 1
			print ""
			print "dn: "dn
			print "changetype: modify"
			print "add: "attr
		}
		print attr": "val
	}'

	rm -f cur_attr_values.tmp
	rm -f desired_attr_values.tmp
	rm -f desired_attrs.tmp2
}

search_compare_fix_propagate(){

	# input
	srchbase=$1
	desired_attrs_fn=$2 # a file name

	ldapsearch $ldapcred -s base -b "$srchbase" "objectclass=*" \
		ownerpropagate aclpropagate 2>cur_errors.tmp | \
		tail +2 | sort >cur_attrs.tmp

	if [ "X`cat cur_attrs.tmp`" = "X" ] || [ "X`cat cur_errors.tmp`" != "X" ];then
		rm -f cur_errors.tmp
		rm -f cur_attrs.tmp
		return
	fi
	rm -f cur_errors.tmp

	# determine what needs to change and generate the LDIF to do it
	diff $desired_attrs_fn cur_attrs.tmp | grep "<" | \
			$AWK -v dn="$srchbase" -v desired_attrs_fn=$desired_attrs_fn '{

		# Find the attr in the desired_attrs_fn file
		search_attr = $1	
		while (getline < desired_attrs_fn){
			split($0,a,"=")
			if (toupper(a[1]) == toupper(search_attr)){
				break
			}
		}
		close(desired_attrs_fn)

		attr = substr($0,1,index($0,"=")-1)
		val = substr($0,index($0,"=")+1)

		print ""
		print "dn: "dn
		print "changetype: modify"
		print "replace: "attr
		print attr": "val
	}'

	rm -f cur_attrs.tmp
}

# Main

# First, check "secauthority=default" suffix

# Check whether secauthority=default already exists.  If so, use the
# ACLs defined on it

ldapsearch $ldapcred -s base -b secauthority=default "objectclass=*" \
	aclentry entryowner aclpropagate ownerpropagate >sec_desired_acls.tmp 2>&1
rc=$?

cat sec_desired_acls.tmp | tail +2 | sort >sec_desired_acls.tmp2
mv sec_desired_acls.tmp2 sec_desired_acls.tmp

if [ $rc = 0 ];then

	# secauthority=default exists, add subdomain acls if they exist

	ldapsearch $ldapcred -s one -b cn=subdomains,secauthority=default \
		"objectclass=*" dn >/dev/null 2>&1 
	rc=$?
	found=`ldapsearch $ldapcred -s one -b cn=subdomains,secauthority=default \
		"objectclass=*" dn 2>&1 | wc -l`

	if [ $rc = 0 ] && [ $found != 0 ];then

		# there are subdomains defined, so add their ACLs to those for secauthority=default
		ldapsearch $ldapcred -s one -b cn=subdomains,secauthority=default "objectclass=*" aclentry | \
			tail +2 | grep -i subdomains >>sec_desired_acls.tmp

		cat sec_desired_acls.tmp | sort | uniq >sec_desired_acls.tmp2
		mv sec_desired_acls.tmp2 sec_desired_acls.tmp
	fi
else
	# secauthority=default does not exist.
	# Define the default ACLS.

	cat << EOF >sec_desired_acls.tmp
aclpropagate=TRUE
ownerpropagate=TRUE
entryowner=group:CN=SECURITYGROUP,SECAUTHORITY=DEFAULT
aclentry=group:CN=REMOTE-ACL-USERS,CN=SECURITYGROUPS,SECAUTHORITY=DEFAULT:normal:rsc:system:rsc:at.secAcctValid:rwsc:at.secPwdFailCountTime:rwsc:at.secPwdFailures:rwsc:at.secPwdLastChanged:rwsc:at.secPwdLastFailed:rwsc:at.secPwdLastUsed:rwsc:at.secPwdUnlockTime:rwsc:at.secPwdValid:rwsc
aclentry=group:CN=SECURITYGROUP,SECAUTHORITY=DEFAULT:object:ad:normal:rwsc:sensitive:rwsc:critical:rwsc:system:rsc
aclentry=group:CN=IVACLD-SERVERS,CN=SECURITYGROUPS,SECAUTHORITY=DEFAULT:normal:rsc:system:rsc:at.secAcctValid:rwsc:at.secPwdFailCountTime:rwsc:at.secPwdFailures:rwsc:at.secPwdLastChanged:rwsc:at.secPwdLastFailed:rwsc:at.secPwdLastUsed:rwsc:at.secPwdUnlockTime:rwsc:at.secPwdValid:rwsc
EOF

	cat sec_desired_acls.tmp | sort >sec_desired_acls.tmp2
	mv sec_desired_acls.tmp2 sec_desired_acls.tmp

fi

# Get the non-secauthority=default suffixes

ignore_suffixes="SECAUTHORITY=DEFAULT|CN=SCHEMA|CN=LOCALHOST|CN=PWDPOLICY|CN=IBMPOLICIES|CN=CONFIGURATION"

ldapsearch -L $ldapcred -s base -b "" "objectclass=*" namingcontexts | \
	$AWK '{
		while ($0 != "" && $1 == "namingcontexts:"){
			attr_tmp = substr($0, index($0,$2))
			$0 = ""
			getline
			while (substr($0,1,1)==" "){
				attr_tmp = attr_tmp""substr($0,index($0,$1))
				getline
			}
			print attr_tmp
		}
	}' | \
	egrep -iv $ignore_suffixes >suffixes.tmp

# check for differences and make recommendations

# use a background pipe
cat suffixes.tmp |&
read -p this_suffix
while [ "X$this_suffix" != "X" ];do

	# check aclentries
	cat sec_desired_acls.tmp | grep -i "aclentry=" >desired_attrs.tmp
	search_compare_fix_attrs "$this_suffix" desired_attrs.tmp aclentry

	# check entryowner
	cat sec_desired_acls.tmp | grep -i "entryowner=" >desired_attrs.tmp
	search_compare_fix_attrs "$this_suffix" desired_attrs.tmp entryowner

	# check all else (propagates)
	cat sec_desired_acls.tmp | egrep -vi 'aclentry=|entryowner=' >desired_attrs.tmp
	search_compare_fix_propagate "$this_suffix" desired_attrs.tmp

	read -p this_suffix
done

rm -f suffixes.tmp
rm -f sec_desired_acls.tmp
rm -f desired_attrs.tmp

