#!/bin/ksh

# Restrictions:
# - This script must be run under the context of the root system user.
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It does not require write authority to the current directory.

if [ `uname` = "SunOS" ];then
        AWK=nawk
else
        AWK=awk
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import

ask_ldapdbinst
stop_ldap	# stop the ldap server

### Database configuration parameter tuning ###

# Set a DB2 database configuration parameter, if not already set
set_db_parm_if_dif(){
	dbparm_name=$1
	recommended_value=$2

	# First, get the current setting
	cur_val=`db2 get database configuration for $ldapdbname | grep $dbparm_name | $AWK '{print $NF}'`

	print ""
	if [ "X$cur_val" = "X$recommended_value" ];then
		print "$dbparm_name is at the recommended setting of $cur_val."
		print "No change is necessary."
	else
		print "$dbparm_name is set to $cur_val, but the recommended setting is $recommended_value."
		echo db2 update database configuration for $ldapdbname using $dbparm_name $recommended_value
		db2 update database configuration for $ldapdbname using $dbparm_name $recommended_value

		dbchanged=1
	fi
}
# initialize
dbchanged=0

# Main db2 tuning

# Ensure that DB2 has been started
db2start >/dev/null

print ""
print "DB2 database configuration parameter tuning"
print ""

# Get the current db config settings
print "The current settings of some key DB2 configuration parameters are as follows:"
print ""
echo "db2 get database configuration for $ldapdbname | egrep 'DBHEAP|SORTHEAP|MAXLOCKS|MINCOMMIT|UTIL_HEAP_SZ|APPLHEAPSZ'"
db2 get database configuration for $ldapdbname | egrep 'DBHEAP|SORTHEAP|MAXLOCKS|MINCOMMIT|UTIL_HEAP_SZ|APPLHEAPSZ'

# Tune the db config settings
print ""
print "Setting the values of some key DB2 configuration parameters, if not already set."

# Need to verify these help before recommending them
#set_db_parm_if_dif SORTHEAP 2500
#set_db_parm_if_dif MAXLOCKS 100
#set_db_parm_if_dif APPLHEAPSZ 2048

# recommend setting MINCOMMIT to 1 if not already set.
#db2 update database configuration for $ldapdbname using MINCOMMIT 1
set_db_parm_if_dif MINCOMMIT 1

if [ $dbchanged = 1 ];then

	# Make sure all connections are terminated for changes to take effect
	echo db2 force applications all
	db2 force applications all
	sleep 2
fi

print ""
print "Press <Enter> to tune the DB2 buffer pools."
read ans

### DB2 buffer pool tuning ####

# Display the memory usage for the current set of buffer pool settings
get_buffer_pool_memory_usage(){

	db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools" | $AWK 'BEGIN{
		l1 = l2 = l3 = sprintf("   ")
		first = 1
	}{
		if (!past_header){
			if (index($0,"-----")){
				past_header = 1
			}
		}else if ($1=="" || skip_to_end){
			skip_to_end = 1
		}else if (first){
			first = 0
			l1 = l1 "" sprintf("%s",$1)
			l2 = l2 "" sprintf("%d*%d",$2,$3)
			l3 = l3 "" sprintf("%3.1f",$2*$3/1024/1024)
			sum += $2*$3
		}else{
			l1 = l1 "" sprintf(" + %s",$1)
			l2 = l2 "" sprintf(" + %d*%d",$2,$3)
			l3 = l3 "" sprintf(" + %3.1f",$2*$3/1024/1024)
			sum += $2*$3
		}
	}END{
		printf("%s = \n",l1)
		printf("%s = %d bytes\n",l2,sum)
		printf("%s = %3.1f MB\n",l3,sum/1024/1024)
	}'
}

# Set a DB2 buffer pool parameter, if not already set
set_bufpool_if_dif(){
	bpname=$1
	recommended_value=$2

	cur_val=`db2 connect to $ldapdbname >/dev/null 2>&1;
		db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools" | 
		grep -i $bpname | $AWK '{print $2}'`

	print ""
	if [ "X$cur_val" = "X$recommended_value" ];then
		print "$bpname is at the recommended size of $cur_val."
		print "No change is necessary."
	else
		print "$bpname is $cur_val, but the recommended size is $recommended_value."

		echo db2 alter bufferpool $bpname size $recommended_value
		db2 alter bufferpool $bpname size $recommended_value

		bpchanged=1
	fi
}

# initialize buffer pool changed flag
bpchanged=0

print "DB2 database buffer pool tuning"
print ""

echo db2 connect to $ldapdbname
db2 connect to $ldapdbname

# Get the current buffer pool settings
print "The current buffer pool sizes are as follows:"
print ""
echo db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools"
db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools"

print "The current buffer pool memory usage is as follows:"
print ""

get_buffer_pool_memory_usage

# Tune the buffer pool

print ""
print "Setting the sizes of the DB2 buffer pools, if not already set."

# Use the TAM recommended default below, uncomment one of the example
# settings or define a new setting.

# IDS 5.1 and 6.0 defaults
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   115.2 + 38.4 = 153.7 MB
# ratio:  3 to 1
#set_bufpool_if_dif ibmdefaultbp 29500
#set_bufpool_if_dif ldapbp 9800

# IDS 6.0 tuning guide example configuration
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   78.1 + 306.3 = 384.4 MB
# ratio:  1 to 3.9
#set_bufpool_if_dif ibmdefaultbp 20000
#set_bufpool_if_dif ldapbp 9800

# TAM recommendation
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   194.5 + 64.8 = 259.4 MB
# ratio:  3 to 1
#db2 alter bufferpool ibmdefaultbp size 49800
#db2 alter bufferpool ldapbp size 2075
#set_bufpool_if_dif ibmdefaultbp 49800
#set_bufpool_if_dif ldapbp 2075

# Halving the TAM recommendation
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   97.3 + 32.4 = 129.7 MB
# ratio:  3 to 1
#set_bufpool_if_dif ibmdefaultbp 24900
#set_bufpool_if_dif ldapbp 1038

# Doubling the TAM recommendation
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   389.1 129.7 = 518.8 MB
# ratio:  3 to 1
#set_bufpool_if_dif ibmdefaultbp 99600
#set_bufpool_if_dif ldapbp 4150

# Proportionally increasing the TAM recommendation
# to use 1 GB of memory
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   768.0 + 256.0 = 1024.0 MB
# ratio:  3 to 1
#set_bufpool_if_dif ibmdefaultbp 196608
#set_bufpool_if_dif ldapbp 8192

# Best so far
# Equal memory to both buffer pools
# using 1 GB of memory
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   512 + 512 = 1024 MB
# ratio:  1 to 1
#set_bufpool_if_dif ibmdefaultbp 131072
#set_bufpool_if_dif ldapbp 16384

# not good performance
# Reverse the original TAM recommended allocation ratio
# using 1 GB of memory
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   256 + 768 = 1024 MB
# ratio:  1 to 3
#set_bufpool_if_dif ibmdefaultbp 65536
#set_bufpool_if_dif ldapbp 24576

# Equal memory to both buffer pools
# using 1 GB of memory
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   384.0 + 640.0 = 1024.0 MB
# ratio:  1 to 1.67
#set_bufpool_if_dif ibmdefaultbp 98304
#set_bufpool_if_dif ldapbp 20480

# Small buffer pools, but 1 to 1 ratio
# TAM recommendation
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   130 + 130 = 260 MB
# ratio:  1 to 1
set_bufpool_if_dif ibmdefaultbp 33200
set_bufpool_if_dif ldapbp 4160

if [ $bpchanged = 1 ];then
	echo ""
	echo db2 terminate
	db2 terminate
	echo db2 force applications all
	db2 force applications all
	sleep 2
	echo db2stop
	db2stop
	echo db2start
	db2start

	# Verify new settings

	echo db2 connect to $ldapdbname
	db2 connect to $ldapdbname
	print "The new buffer pool sizes are as follows:"
	print ""
	echo db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools"
	db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools"

	print "The new buffer pool memory usage is as follows:"
	print ""
	get_buffer_pool_memory_usage
	print ""

	echo db2 terminate
	db2 terminate
fi

print ""
print "Press <Enter> to tune the transaction logs."
read ans

### DB2 transaction log tuning

askif "Describe the DB2 transaction log tuning" y
if [ $doit = 1 ];then

print DB2 transaction log space is defined by the LOGFILSIZ, LOGPRIMARY, LOGSECOND, NEWLOGPATH
print parameters.  These parameters should be tuned to allow the transaction log to grow
print to its maximum required size.  In normal use of the IBM Tivoli Directory Server, the
print transaction log requirements are small.  Tools that improve the performance of populating
print the directory server with a large number of users typically increase the transaction log
print requirements for a short period of time.  Here are examples:
print ""
print - The bulkload tools load attribute tables for many entries in a single load command.
print   One table of particular interest is the group membership table.  Bulkloading a
print   large group of millions of users will increase the transaction log requirements.
print ""
print   The transaction log requirements to load a 3 million user group is around 300 MB.
print ""
print   Note: IDS version 6.0 includes a new option on bulkload that allows the bulkload
print   of a group to be split into multiple loads.  This option can be used to reduce the
print   transaction log requirements for loading a static group.
print ""
# check on this:
# - The Access Manager tuning guide scripts can update the ACL on a suffix such that
#   the ACL must be propagated to many other entries in the directory.  The IBM directory
#   server combines all of the propagated updates into a single committed transaction.
print - An ACL placed on a suffix object can result in that ACL propogating to every
print   entry under the suffix.  The IBM directory server performs ACL propagation as one
print   single committed DB2 transaction.
print ""
print   The transaction log requirements to propagate ACLs to a suffix with 3 million
print   Access Manager users in the TAM 5.1 LDAP data model is around 1.2 GB.
print ""
print Using the 1.2 GB requirement above, the transaction log requirements are approximately
print ""
print 1200000000 bytes / 3000000 users = 400 bytes per user
print ""
print The DB2 defaults define each transaction log file to be 2000 blocks of 4KB in size
print or 8000 KB.  This script changes the default number of blocks per file to 5000 which uses
print approximately 20 MB of disk space per transaction log file.
print ""
print The default number of primary log files is 3 and secondary log files is 2.  This script
print sets the number primary log files to its default of 3 and sets the number of secondary
print log files to be sufficient for the 3 milion TAM user in the TAM 5.1 data model case.
print ""
print "With the default settings made by this script, the primary log files will use (20MB * 3)"
print "or 60 MB of disk space and the secondary log files can grow to a maximum of (20MB * 47) or"
print 1.1 GB or disk space.
print ""
print If adjustments are required to these settings, it is recommended that they be made to the
print number of secondary log files, instead of the number of primary log files.  This is because
print the file space for the secondary log files is recovered at various times, including when db2
print "is restarted print (db2stop/db2start)."
print ""
print Using the 400 bytes per user requirements from the ACL propagation case and the number of
print primary logs and size of the log file set by this script, the formula for increasing the
print number of transaction log secondary files is as follows:
print ""
print "( ( <num AM users> * 400 ) - ( 20MB file size * 3 primary log files ) ) / 20 MB file size"
print ""
print For 3 million users, this approximates to the following:
print ""
print "( ( 3 * 400 ) - ( 20 * 3  ) ) / 20 = 57 secondary log files"
print ""
print The disk space requirements for this number of files is 1.2 GB
print "          20MB * ( 3 + 57 ) = 1.2 GB"

if [ 0 = 1 ];then

print ""
print "Defining the transaction log size parameters to allow for the worst case of ACL"
print "propagation.  The chosen setting will allow ACLs to propagate from a suffix to"
print "up to 3 million Access Manager users.  This setting can use up to 1.2GB of additional"
print "disk space in the DB2 instance owner's home directory."
print ""
print "The number 4KB blocks per log file will be increased to 5000."
print "The number of secondary log buffers will be increased to 57."

fi

print ""
print Adjust the LOGSECOND and LOGFILSIZ values in this script for more or less users.
print ""
print WARNING: Ensure there is enough disk space for whatever setting is used, since running out
print transaction log disk space can corrupt the database and require it to be reloaded or restored
print from a backup.  It is better to define an too little space in the DB2 configuration paramters
print for the transaction logs than it is to not have enough space on the files system for the
print configured parameters.  Regular backups are a good preventative measure, in case the database
print becomes corrupted.

print ""
print "Press <Enter> to continue database tuning."
read ans

fi

# initialize
dbchanged=0

# Get the current settings
print "The current transaction log settings are as follows:"
print ""
echo db2 get database configuration for $ldapdbname | egrep 'LOGFILSIZ|LOGPRIMARY|LOGSECOND|NEWLOGPATH|Path to log files'
db2 get database configuration for $ldapdbname | egrep 'LOGFILSIZ|LOGPRIMARY|LOGSECOND|NEWLOGPATH|Path to log files'

print ""
print "Setting the transaction log values, if not already set."

set_db_parm_if_dif LOGPRIMARY 3
# for 3 million
set_db_parm_if_dif LOGFILSIZ 5000
# for 30 million
# set_db_parm_if_dif LOGFILSIZ 10000

# Note: Update this parameter to increase or decrease the transaction log space.
# for 3 million
set_db_parm_if_dif LOGSECOND 57
# for 30 million
#set_db_parm_if_dif LOGSECOND 250

if [ $dbchanged = 1 ];then

	# Make sure all connections are terminated for changes to take effect
	print ""
	echo db2 force applications all
	db2 force applications all
	sleep 2

	print ""
	print "The new transaction log settings are as follows:"
	print ""
	echo db2 get database configuration for $ldapdbname | egrep 'LOGFILSIZ|LOGPRIMARY|LOGSECOND|NEWLOGPATH|Path to log files'
	db2 get database configuration for $ldapdbname | egrep 'LOGFILSIZ|LOGPRIMARY|LOGSECOND|NEWLOGPATH|Path to log files'
fi

print ""
print "Verifying that the file system for the transaction logs is large enough to"
print "accommodate the maximum growth."
print ""

print "Filesystem limits"
echo "logfilpath=\`db2 get db cfg for $ldapdbname | grep "Path to log files" | $AWK '{print \$NF}'\`"
echo "logfilpath="`db2 get db cfg for $ldapdbname | grep "Path to log files" | $AWK '{print $NF}'`
logfilpath=`db2 get db cfg for $ldapdbname | grep "Path to log files" | $AWK '{print $NF}'`

echo "df -k $logfilpath"
df -k $logfilpath
fs_check=`df -k $logfilpath | $AWK 'BEGIN{getline}{print $0}'`

if [ `uname -s` = SunOS ];then
	echo "avail=\`echo $fs_check | $AWK '{print \$4 / 1024 }'\`"
	echo "avail="`echo $fs_check | $AWK '{print $4 / 1024 }'`
	avail=`echo $fs_check | $AWK '{print $4 / 1024 }'`
else
	echo "avail=\`echo $fs_check | $AWK '{print \$3 / 1024 }'\`"
	echo "avail="`echo $fs_check | $AWK '{print $3 / 1024 }'`
	avail=`echo $fs_check | $AWK '{print $3 / 1024 }'`
fi

print ""
print "Database log space limits"
echo "logfilsiz=\`db2 get database configuration for $ldapdbname | grep LOGFILSIZ | $AWK '{print \$NF}'\`"
echo "logfilsiz="`db2 get database configuration for $ldapdbname | grep LOGFILSIZ | $AWK '{print $NF}'`
logfilsiz=`db2 get database configuration for $ldapdbname | grep LOGFILSIZ | $AWK '{print $NF}'`

echo "logprimary=\`db2 get database configuration for $ldapdbname | grep LOGPRIMARY | $AWK '{print \$NF}'\`"
echo "logprimary="`db2 get database configuration for $ldapdbname | grep LOGPRIMARY | $AWK '{print $NF}'`
logprimary=`db2 get database configuration for $ldapdbname | grep LOGPRIMARY | $AWK '{print $NF}'`

echo "logsecond=\`db2 get database configuration for $ldapdbname | grep LOGSECOND | $AWK '{print \$NF}'\`"
echo "logsecond="`db2 get database configuration for $ldapdbname | grep LOGSECOND | $AWK '{print $NF}'`
logsecond=`db2 get database configuration for $ldapdbname | grep LOGSECOND | $AWK '{print $NF}'`

echo "logspace=( logfilsiz * 4096 * ( logprimary + logsecond ) / 1024 / 1024 )"
echo "logspace=( $logfilsiz * 4096 * ( $logprimary + $logsecond ) / 1024 / 1024 )"
echo "logspace="$(( $logfilsiz * 4096 * ( $logprimary + $logsecond ) / 1024 / 1024 ))
logspace=$(( $logfilsiz * 4096 * ( $logprimary + $logsecond ) / 1024 / 1024 ))

print ""
print "The available file space on $logfilpath is $avail MB."
print "The log parameters allow the log files to grow to a maximum of $logspace MB."
print ""

if [ $avail -lt $logspace ];then
	print "Warning: The maximum allowed log space exceeds the available log space."
	print "Consider increasing the storage in the file system or decreasing the"
	print "log space parameters.  Running out of file space for transaction logs"
	print "can corrupt the database and require it to be restored."
else
	print "Check succeeded:  There is sufficient disk space to allow for the maximum log"
	print "file growth."
fi
print ""

