#!/bin/ksh

# Script to fix up ACLs after IDS bulkload has been performed without ACLs
# either through the use of the "-A no" bulkload option or when the loaded
# LDIF does not contain ACLs.  It is recommended when using the old TAM
# data format and over 100,000 TAM users are to be loaded, that ACLs are
# not generated in the loaded LDIF.  Note that a previously loaded LDAP
# directory server without ACLs will not have ACLs in the output from
# db2ldif, even if some version of the fixacls TAM tuning guide script
# has been run.  In these case, this script should be run to fix up
# the ACLs to provide the ACL protections recommended by TAM.
#
# Note that the strong ACL protections recommended by TAM may not be
# necessary in all cases, for example, if the directory server is
# restricted to the root administrator or physically restricted in some
# way.  In these cases, it may be sufficient to place an restrictive
# ACL on each LDAP suffix, instead of using this script.
#
# This script first determines whether a usage of IDS bulkload has
# restulted in LDAP entries with undefined ACLs.  Any LDAP entry with
# undefined ACL is fixed up to inherit its ACL and entry ownership from
# its parent entry following the definition of the ACLPROPAGATE and
# OWNERPROPAGATE attributes.
#
# Next this script fixes up the ACLs on the entire subtree of any LDAP
# entry that has a secAuthority attribute and is not the source of its
# own ACL.  In other words, the LDAP entry does not have an ACL defined
# to it.  The ACL is fixed up to inherit from the appropriate secAuthInfo
# LDAP entry for the domain identified in the secAuthority attribute.
#
# Note that this script may have high DB2 transaction log requirements.
# It may be necessary to increase the transaction log space.  See the
# db2_tunings.sh TAM tuning guide script for more information.
#
# Restrictions:
# - This script must be run under the context of the root system user.
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It requires write authority to the current directory.

if [ `uname` = "SunOS" ];then
        AWK=nawk
else
        AWK=awk
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import

ask_ldapdbinst

echo ""
echo "Start of $0: "`date`
echo ""

# Stop the directory server
stop_ldap

# Ensure db2 is using indexes

get_db2_level
supports_volatile=1
if [ $db2vers -lt 8 ];then
	supports_volatile=0
elif [ $db2fp -lt 8 ];then
	supports_volatile=0
fi

if [ $supports_volatile = 1 ];then
	echo db2_volatile_tune.sh
	db2_volatile_tune.sh
else
	echo sysstat_tune_small_tables.sh
	sysstat_tune_small_tables.sh
fi

# Disable the system statistic tunes, so the the DESC (descendant)
# table will be used.
echo sysstat_tune.sh disable
sysstat_tune.sh disable

# Connect to the database
echo "db2 connect to $ldapdbname >/dev/null"
db2 connect to $ldapdbname >/dev/null
echo ""

print_if_not_empty(){
#	grep SQL0100W update.tmp >/dev/null
#	rc1=$?
#	grep DB20000I update.tmp >/dev/null
#	rc2=$?
#	if [ $rc1 = 1 ] && [ $rc2 = 1 ];then
#		cat update.tmp
#	fi
	cat update.tmp | $AWK '{print "\t"$0}'
}

print_dn_trunc(){

	local_eid=$1
	db2 -x "select dn_trunc from $tabschema.ldap_entry where eid = $local_eid" | \
		$AWK '{
			for (i=length($0);i&&substr($0,i,1)==" ";i--){}
			if ($1=="") {exit}
			print substr($0,1,i)
		}' 
}

get_secauth_suffix_eid(){

	if [ X$secauth_suffix = X ];then
		# Get the secauthority=default object, so it can be used later
		echo "Obtaining the eid for the secAuthority=Default suffix"
		echo "\tdb2 -x \"select eid from $tabschema.ldap_entry where dn_trunc = 'SECAUTHORITY=DEFAULT'\""
		db2 -x "select eid from $tabschema.ldap_entry where dn_trunc = 'SECAUTHORITY=DEFAULT'" >query_result.tmp
		query_result=`cat query_result.tmp | $AWK '{print $1}'`
		print "\t$query_result"

		secauth_suffix=$query_result
	fi
}

find_all_domains(){

	if [ X$domains = X ];then
		# First find all domains
		print "Finding all Access Manager subdomains : "`date`
		echo "\tdb2 -x select distinct secauthority from $tabschema.secauthority"
		db2 -x select distinct secauthority from $tabschema.secauthority >query_result.tmp
		cat query_result.tmp | $AWK '{print substr($0,index($0,$1))}' >domains.tmp
		for i in `cat domains.tmp`;do
			print "\t$i"
		done

		if [ "X`cat domains.tmp`" != X ];then
			domains=1
		fi
	fi
}


# First determine if there are any LDAP entries with undefined ACLs and
# fix them up.  These are called bulkload objects below, because an IDS bulkload
# versions previous to 6.0 could produce them.

fixup_done=0
found_end_last_time=0
last_range_end=0

# Find the range of the bulkloaded objects
getrange(){

if [ $found_end_last_time = 1 ];then
	fixup_done=1
else

	print "Looking for LDAP entries with an undefined ACL : "`date`
	print "  This can take a long time depending upon the size of the directory server..."

	# Get the EID of the first bulkloaded object
	# (skip over suffixes)
	print ""
	print "Obtaining the minimum eid from all LDAP entries that have an undefined ACL"
	cat <<EOF
	db2 -x "select min(eid) from $tabschema.src where \\
		aclsrc = -1 and \\
		eid > $last_range_end and \\
		not eid in (select eid from $tabschema.ldap_entry \\
		where peid = -1)"
EOF
	db2 -x "select min(eid) from $tabschema.src where \
		aclsrc = -1 and \
		eid > $last_range_end and \
		not eid in (select eid from $tabschema.ldap_entry \
		where peid = -1)" >query_result.tmp
	query_result=`cat query_result.tmp | $AWK '{print $1}'`

	if [ "X$query_result" = "X-" ];then
		print "\tNone found."
		# no bulkloaded objects
		fixup_done=1
	else
		print "\t$query_result"
		range_start=$query_result

		# Get the EID of the next non-bulkloaded object or
		# the last one, which is more likely the case

		echo "Obtaining the eid of the next LDAP entry with a defined ACL"
		cat <<EOF
	db2 -x "select min(eid) from $tabschema.src where \\
		not aclsrc = -1 and eid > $range_start"
EOF
	db2 -x "select min(eid) from $tabschema.src where \
		not aclsrc = -1 and eid > $range_start" >query_result.tmp
		query_result=`cat query_result.tmp | $AWK '{print $1}'`

		# If not found, then it is the end
		if [ "X$query_result" = "X" ];then

			print "\tNone found."
			found_end_last_time=1

			# Get the last EID
			print ""
			print "Obtaining the last eid"
			echo "\tdb2 -x \"select max(eid) from $tabschema.ldap_entry\""
			db2 -x "select max(eid) from $tabschema.ldap_entry" >query_result.tmp
			query_result=`cat query_result.tmp | $AWK '{print $1}'`
			echo "\t$query_result"
			range_end=$query_result 

		else
			# Set the end to what was found
			range_end=$(( query_result - 1 ))
			last_range_end=$range_end
			print "\t$range_end"
		fi
	fi
fi

}

# Function for finding the parent LDAP entry to use for inheritance (either ACL or owner)
#
# Besides command line inputs, this also requires the "parent" variable to be set
# Returns the "parent_for_inheritance" variable
find_parent_for_inheritance(){
	attr_printed_name=$1
	propagation_attr_db_name=$2

	parent_for_inheritance=$parent
	done=0
	while [ $done = 0 ];do

		echo "Obtaining the $attr_printed_name propagation flag for the parent LDAP entry."
		echo "Null means true."

		echo "\tdb2 -x select $propagation_attr_db_name from $tabschema.$propagation_attr_db_name where eid = $parent_for_inheritance"
		db2 -x select $propagation_attr_db_name from $tabschema.$propagation_attr_db_name where eid = $parent_for_inheritance >query_result.tmp
		query_result=`cat query_result.tmp | $AWK '{print $1}'`
		echo "\t$query_result"

		# get the last result - really shouldn't have more than one,
		# but have seen it once.
		query_result=`print $query_result | $AWK '{print $NF}'`

		# apparently, null means true
		if [ "X$query_result" = X1 ] || [ "X$query_result" = X ];then
			done=1
		else
			# find the parent of the parent or the root
			echo "Obtaining the parent eid of this parent"
			echo "\tdb2 -x select peid from $tabschema.ldap_entry where eid = $parent_for_inheritance"
			db2 -x "select peid from $tabschema.ldap_entry where eid = $parent_for_inheritance" >query_result.tmp
			query_result=`cat query_result.tmp | $AWK '{print $1}'`
			echo "\t$query_result"
	
			parent_for_inheritance=$query_result
			if [ "$parent_for_inheritance" = "-1" ];then
				done=1
			fi
		fi
	done
}

# Main

# Get the EID range of the next bulkloaded objects
getrange

#DEBUG print "range_start: "$range_start" range_end: "$range_end" fixup_done: "$fixup_done" found_end_last_time: "$found_end_last_time

while [ $fixup_done = 0 ];do

# Get the parents of the bulkloaded objects
print "Finding the parent eids of all LDAP entries with an undefined ACL, i.e. where aclsrc = -1 : "`date`

cat << EOF
	db2 -x "select distinct peid from $tabschema.ldap_entry \\
		where eid >= $range_start and \\
		eid <= $range_end and \\
		peid < $range_start \\
		peid > -1"
EOF
db2 -x "select distinct peid from $tabschema.ldap_entry \
	where eid >= $range_start and \
	eid <= $range_end and \
	peid < $range_start and \
	peid > -1" >query_result.tmp
query_result=`cat query_result.tmp | $AWK '{print $1}'`

parent_eids=$query_result
if [ X"$parent_eids" = X ];then
	# this should not happen
	print "\tNone was found."
else
	for i in $parent_eids;do
		print "\t$i"
	done

	# Reverse the parent eids, so that the deepest parent in the directory tree
	# is fixed up first
	parent_eids=`print $parent_eids | $AWK '{for (i=NF;i;i--){print $i}}'`

	for parent in $parent_eids;do

		# print out the parent
		print ""
		print "Propagating ACLs to the children of the following parent LDAP entry : "`date`
		print -n "\t"
		print_dn_trunc $parent

		# Find the children of this parent
		# This prints out the children eids (for debug purposes)
		# print "parent: "$parent
		# db2 -x "select deid from $tabschema.ldap_desc where aeid = $parent and not aeid = deid"

		# Find a parent that has aclpropagate = true
		# in case the direct parent does not propagate

		print ""
		print "Finding the nearest parent LDAP entry that has aclpropagate = true"
		print "That will be the entry from which to inherit the ACL"
		print ""

		find_parent_for_inheritance ACL aclprop

		if [ "$parent_for_inheritance" != "-1" ];then

			print "Propagating the ACLs (source and type) from the following parent LDAP entry : "`date`
			print -n "\t"
			print_dn_trunc $parent_for_inheritance

			print ""
			print "Obtaining the ACL source of the parent LDAP entry"
			echo "db2 -x select aclsrc from $tabschema.src where eid = $parent_for_inheritance"
			db2 -x select aclsrc from $tabschema.src where eid = $parent_for_inheritance >query_result.tmp
			query_result=`cat query_result.tmp | $AWK '{print $1}'`
			echo "\t$query_result"

			if [ "$query_result" = "-1" ];then
				print "No ACL to inherit from the suffix.  No change necessary."
			else

				# fix up the aclsrc and acltype of the just bulkloaded children
				# of this parent
				cat << EOF
	db2 -x "update $tabschema.src set ( aclsrc, acltype ) = \\
		(select aclsrc, acltype from \\
		$tabschema.src as p where p.eid = $parent_for_inheritance) where \\
		eid >= $range_start and eid <= $range_end and \\
		eid in (select deid from $tabschema.ldap_desc where \\
		aeid = $parent_for_inheritance and \\
		deid >= $range_start and \\
		deid <= $range_end)"
EOF
				db2 -x "update $tabschema.src set ( aclsrc, acltype ) = \
					(select aclsrc, acltype from \
					$tabschema.src as p where p.eid = $parent_for_inheritance) where \
					eid >= $range_start and eid <= $range_end and \
					eid in (select deid from $tabschema.ldap_desc where \
					aeid = $parent_for_inheritance and \
					deid >= $range_start and \
					deid <= $range_end)" >update.tmp
				print_if_not_empty
			fi

		fi

		# Find a parent that has ownerpropagate = true
		# in case the direct parent does not propagate

		print ""
		print "Finding the nearest parent LDAP entry that has ownerpropagate = true"
		print "That will be the entry from which to inherit the entry owner"
		print ""

		find_parent_for_inheritance owner ownerprop

		if [ "$parent_for_inheritance" != "-1" ];then

			print ""
			print "Obtaining the entry owner of the parent LDAP entry"
			echo "db2 -x select ownsrc from $tabschema.src where eid = $parent_for_inheritance"
			db2 -x select ownsrc from $tabschema.src where eid = $parent_for_inheritance >query_result.tmp
			query_result=`cat query_result.tmp | $AWK '{print $1}'`
			echo "\t$query_result"

			if [ "$query_result" = "-1" ];then
				print "No owner to inherit from the suffix.  No change necessary."
			else

				print "Propagating the entry owner (ownsrc) from the following parent LDAP entry : "`date`
				print -n "\t"
				print_dn_trunc $parent_for_inheritance

				# fix up the ownsrc of the just bulkloaded children
				# of this parent
				cat << EOF
	db2 -x "update $tabschema.src set ownsrc = \\
		(select ownsrc from \\
		$tabschema.src as p where p.eid = $parent_for_inheritance) where \\
		eid >= $range_start and eid <= $range_end and \\
		eid in (select deid from $tabschema.ldap_desc where \\
		aeid = $parent_for_inheritance and \\
		deid >= $range_start and \\
		deid <= $range_end)"
EOF
				db2 "update $tabschema.src set ownsrc = \
					(select ownsrc from \
					$tabschema.src as p where p.eid = $parent_for_inheritance) where \
					eid >= $range_start and eid <= $range_end and \
					eid in (select deid from $tabschema.ldap_desc where \
					aeid = $parent_for_inheritance and \
					deid >= $range_start and \
					deid <= $range_end)" >update.tmp
				print_if_not_empty
			fi
		fi

	done
fi

# Get the EID range of the next bulkloaded objects
print ""
getrange

done


# Next, fix up the ACLs on any LDAP entry that has a secAuthority
# attribute and is not the source of its own ACL.  The ACL is fixed
# up to inherit from the appropriate secAuthInfo LDAP entry for the
# domain identified in the secAuthority attribute.

cat << EOF

Fixing up the ACLs on any Access Manager user entry that is not in the minimal
TAM LDAP data format (prior to verion 6.0) and that does not have an ACL already
defined to it (i.e. its ACL is inherited from some other entry).

EOF

# Make sure we have some info from the ldap db2 database
get_secauth_suffix_eid
find_all_domains

# if there is at least one domain defined
if [ X$domains = X ];then

	print "TAM is not configured into this directory server."
else

	for domain in `cat domains.tmp`;do

		print "Fixing up ACLs of entries in the $domain domain : "`date`
		print "These entries will be set to inherit their ACLs from the following entry:"
		if [ $domain = "DEFAULT" ]; then
			print "\tsecAuthority=Default"
		else
			print "\tsecAuthority=$domain,cn=subdomains,secAuthority=Default"
		fi

		# Determine the eid of the object from which to inherit
		if [ $domain = "DEFAULT" ]; then

			am_acl_src_eid=$secauth_suffix
		else

			echo "Obtaining the eid of the secAuthority=$domain,cn=subdomains,secAuthority=Default entry"
			cat << EOF
	db2 -x "select eid from $tabschema.ldap_entry where \\
		dn_trunc = 'SECAUTHORITY=$domain,CN=SUBDOMAINS,SECAUTHORITY=DEFAULT'"
EOF
			db2 -x "select eid from $tabschema.ldap_entry where \
				dn_trunc = 'SECAUTHORITY=$domain,CN=SUBDOMAINS,SECAUTHORITY=DEFAULT'" >query_result.tmp
			query_result=`cat query_result.tmp | $AWK '{print $1}'`
			echo "\t$query_result"
			
			am_acl_src_eid=$query_result
		fi

		if [ X$am_acl_src_eid != X ];then

			# This update can be very slow if there are many entries in the directory server.
			# It scans at every TAM user entry determining if it needs to be fixed up.

			print "Updating the ACL source of any Accesss Manager user entries that are not correct : "`date`
			print "This can take a long time if the directory is very large."

if [ 1 = 1 ];then
			# This sql is summarized as follows.  Update any ACL src to am_acl_src_eid if it is
			# not already that value and it is the descendant of an entry that either has the
			# src of am_acl_src_eid or is not the owner of its own ACL (eid <> aclsrc) and the
			# ancestor has secauthority of the domain of interest and the ancestor is not the
			# am_acl_src_eid entry.

			cat <<EOF
	db2 "update $tabschema.src set aclsrc = $am_acl_src_eid where \\
		aclsrc <> $am_acl_src_eid and \\
		eid in (select deid from $tabschema.ldap_desc where \\
			aeid in (select eid from $tabschema.src where \\
				(aclsrc = $am_acl_src_eid  or \\
					eid <> aclsrc \\
				) and \\
				eid in (select eid from $tabschema.secauthority where \\
					secauthority = '$domain' \\
				) and \\
				eid <> $am_acl_src_eid \\
			) \\
		)"
EOF
			db2 "update $tabschema.src set aclsrc = $am_acl_src_eid where \
				aclsrc <> $am_acl_src_eid and \
				eid in (select deid from $tabschema.ldap_desc where \
					aeid in (select eid from $tabschema.src where \
						(aclsrc = $am_acl_src_eid or \
							eid <> aclsrc \
						) and \
						eid in (select eid from $tabschema.secauthority where \
							secauthority = '$domain' \
						) and \
						eid <> $am_acl_src_eid \
					) \
				)" >update.tmp
			print_if_not_empty
fi
if [ 0 = 1 ];then	# This was slower

			# Update if this object is or is a descendant of a secauthority entry and
			# its ACL src is not from a secauthority entry.

			cat <<EOF
	db2 "update $tabschema.src set aclsrc = $am_acl_src_eid where \\
		aclsrc <> $am_acl_src_eid and \\
		eid in (select deid from $tabschema.ldap_desc where \\
			aeid in (select eid from $tabschema.secauthority where \\
				secauthority = '$domain' \\
			) \\
		) and \\
		aclsrc not in (select eid from $tabschema.secauthority)"
EOF
			db2 "update $tabschema.src set aclsrc = $am_acl_src_eid where \
				aclsrc <> $am_acl_src_eid and \
				eid in (select deid from $tabschema.ldap_desc where \
					aeid in (select eid from $tabschema.secauthority where \
						secauthority = '$domain' \
					) \
				) and \
				aclsrc not in (select eid from $tabschema.secauthority)" >update.tmp
			print_if_not_empty

fi
		fi
	done
fi

rm -f query_result.tmp
rm -f update.tmp
rm -f domains.tmp
db2 terminate >/dev/null

# Enable the system statistic tunes for best runtime performance
echo sysstat_tune.sh
sysstat_tune.sh

echo ""

echo "End of $0: "`date`

