#!/bin/ksh

# Script to automate the importing of db2ldif data from another
# directory server.  This script assumes the directory server
# into which data is to be imported is empty, but tuned for
# a large number of updates (see am_tune_ldap.sh).

# Migration Requirements:
#   Perform the following steps on the machine upon which this script
#   is run:
#   - Copy the LDIF file obtained from running db2ldif on the previous version
#     LDAP server.  Using the "-j" option on db2ldif is recommended.
#   - Install and configure LDAP.
#   - Define in the LDAP configuration file the suffixes that will be imported
#     (e.g those that exist in the db2ldif output file).
#   Note: Do not use ldapadd or the DMT tool to create the suffix objects,
#     unless those objects are corresponding removed from the LDIF input file.
#   - Apply the Access Manager schema to the sever with the new LDAP version.
#   - Customize the environment variables in the first part of this script
#     to reference the appropriate file, directories, and options to
#     be used.  See the section starting with "BEGIN customize section".
#   - Make sure there is enough disk space and memory to meet the requirements
#     of the bulkload scripts and the directory being migrated.  Refer to the
#     Access Manager Tuning Guide for bulkload requirements.
#   - Make sure the ldapdb2 home directory has enough disk space to meet the
#     transaction log requirements of this script.  Here is a formula:
#         <# of Access Manager users> * 400
#     For 3 million users this is around 1.2 GB of disks space.
#
#   It is recommended, although not required, that any tunings to the
#   ibmslapd.conf file also be done at this time.  Refer to the
#   tuning guide for more information on tuning ibmslapd.conf.

usage(){
	print "Usage:  $0 <ldif file> <ldap password>"
	print "  where"
	print "    <ldif file> is the name of the file containing the output of db2ldif with the -j option"
	print "    <ldap pasword> is the LDAP server administrator password"
	print "        assuming cn=root is the administrator account"
	exit 1
}

if [ X$1 = X ] || [ X$2 = X ];then
	usage
else
	before_ldif=$1
	export ldap_pwd=$2
fi


# Make sure "." is in the path.
export PATH=.:$PATH

# BEGIN customize section

# Setup up parameters (customize these)

#    The following variable controls how ACLs are loaded.  If set to 0, ldapmodify(add)
#    is used to create all ACLs.  If set to 1, a fixacls script is used to create ACLs on
#    Access Manager user objects and ldapmodify is used to create the ACLs for all other
#    objects.  The fixacls method is faster, but does not get properly represented in a
#    db2ldif dump.  If fixacls is used, it must be reapplied on future migrations.
#    The fixacls method is probably only necessary on LDAP directories of more than around
#    100K Access Manager users.
use_fixacls=1

#    The following variable controls whether bulkload is used on groups.
#    If set to 0, ldapadd is used to load groups.  If set to 1, bulkload is used
#    to load groups.  The main disadvantage to using bulkload for groups is the
#    virtual memory size of the bulkload process when handling very large groups,
#    for example groups with millions of users.  Bulkloading of groups has been
#    tested up to 3M users in a group, where the bulkload process grows to 1GB.
use_bulkload_for_groups=1

#    The following variable controls detailed logging.  If enabled, the output
#    of most commands is put to the log file
detailed_logging=1

#    The following variable controls using db2ldif2 at the end of the
#    migration to extract the migrated directory for verification purposes.
#    For example, to compare against the source LDIF.
use_db2ldif_at_end=1   # setting this to 0 disables use of db2ldif at the end

# END customize section

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

logfile=migrate.out
rm -f $logfile

if [ $detailed_logging = 1 ];then
	logcmd="tee -a $logfile"
else
	logcmd="cat"
fi


# Load the data

echo $0: Start the directory server : `date` 2>&1 | tee -a $logfile
if [ X"`ps -ef | grep slapd | grep -v grep`" = X ];then
	#for IDS 4.1: slapd 2>&1 | $logcmd # ldap must be running to use incremental_group.sh
	# note that LDAP_MAXCARD=NO disables the persistent sysstat tunes in LDAP 5.2.
	LDAP_MAXCARD=NO ibmslapd 2>&1 | $logcmd # ldap must be running to use incremental_group.sh
	while [ `ps -ef -o pcpu | grep -v COMMAND | $AWK '{s+=$1}END{printf("%d",s)}'` -gt 4 ];do
		sleep 1
	done
fi

# Load the suffix objects

echo $0: Load the suffix objects : `date` 2>&1 | tee -a $logfile
cat $before_ldif | extract_suffixes.sh | \
	ldapadd -D cn=root -w $ldap_pwd 2>&1 | $logcmd

# Stop the server
echo $0: Stop the directory server : `date` 2>&1 | tee -a $logfile
if [ X`ps -ef | grep slapd | grep -v grep | $AWK '{print $2}'` != X ];then
	kill -9 `ps -ef | grep slapd | grep -v grep | $AWK '{print $2}'`
fi
while [ X`ps -ef | grep slapd | grep -v grep | $AWK '{print $2}'` != X ];do
	sleep 1
done

# extract the non-group definitions and bulkload them
echo $0: Run incremental_bulkload.sh : `date` 2>&1 | tee -a $logfile
cat $before_ldif | exclude_suffixes.sh | exclude_groups.sh | incremental_bulkload.sh with_indexes
if [ $? != 0 ];then
	echo $0: Exiting early due to errors.
	exit 1
fi

if [ $use_bulkload_for_groups = 1 ];then
	echo $0: Run incremental_bulkload.sh to load groups : `date` 2>&1 | tee -a $logfile
	mv inc_bulk.out inc_bulk_users.out # save the output from the first bulkload
	# extract 10000 lines or the next DN, whichever comes last
	# pass these lines incrementally to incremental_bulkload.sh.
	rm -f group.ldif
	cat $before_ldif | exclude_suffixes.sh | extract_groups.sh | $AWK '{
		if (lncnt++ > 10000 && $1 == "dn:"){
			close("group.ldif")
			system("cat group.ldif | incremental_bulkload.sh with_indexes")
			system("mv inc_bulk.out inc_bulk_groups"++nloads".out")
			system("rm -f group.ldif")
			lncnt = 0
			atleast1dn=0
		}
		if ($1 == "dn:"){
			atleast1dn=1
		}
		print $0 >> "group.ldif"
	}END{
		if (atleast1dn){
			close("group.ldif")
			system("cat group.ldif | incremental_bulkload.sh with_indexes")
			system("mv inc_bulk.out inc_bulk_groups"++nloads".out")
			system("rm -f group.ldif")
		}
	}'
	rm -f group.ldif
fi

# Do some tunings after a large update
echo $0: Do a reorgchk : `date` 2>&1 | tee -a $logfile
su - ldapdb2 -c "db2 connect to ldapdb2;db2 reorgchk update statistics on table all;db2 terminate" 2>&1 | $logcmd
# The following tune was removed because it causes problems with ldapadd for ACLs.
# The tune is done after the ldapadd for ACLs.
#su - ldapdb2 -c "$PWD/sysstat_tune.sh" 2>&1 | $logcmd

# do fixacls (necessary step with Access Manager bulkload scripts)
if [ $use_fixacls = 1 ];then
	#echo $0: Run fixacls_migrate.sh : `date` 2>&1 | tee -a $logfile
	#su - ldapdb2 -c "$PWD/fixacls_migrate.sh" 2>&1 | $logcmd
	echo $0: Run fixacls_propagate_parents_once.sh: `date` 2>&1 | tee -a $logfile
	su - ldapdb2 -c "$PWD/fixacls_propagate_parents_once.sh" 2>&1 | $logcmd
fi 

# Start the director server
echo $0: Start the directory server : `date` 2>&1 | tee -a $logfile
#for IDS 4.1: slapd 2>&1 | $logcmd # ldap must be running to use incremental_group.sh
# note that LDAP_MAXCARD=NO disables the persistent sysstat tunes in LDAP 5.2.
LDAP_MAXCARD=NO ibmslapd 2>&1 | $logcmd # ldap must be running to use incremental_group.sh
while [ `ps -ef -o pcpu | grep -v COMMAND | $AWK '{s+=$1}END{printf("%d",s)}'` -gt 4 ];do
	sleep 1
done
# incremental_group.sh also requires the following check to succeed
while [ X`ps -ef | grep slapd | $AWK '{if ($1 == "ldap")print $2}'` = X ];do
	sleep 1
done

if [ $use_bulkload_for_groups = 0 ];then

	# extract the group definitions (with ACLs) and ldapadd them
	echo $0: Run incremental_group.sh : `date` 2>&1 | tee -a $logfile
	cat $before_ldif | exclude_suffixes.sh | extract_groups.sh | incremental_group.sh
fi

if [ $use_fixacls = 1 ];then
	unset extract_acl_opt
else
	extract_acl_opt=all
fi

# extract non-AM ACLs and ldapadd them
echo $0: Run ldapadd to apply ACLs : `date` 2>&1 | tee -a $logfile
if [ $use_bulkload_for_groups = 0 ];then
	cat $before_ldif | exclude_suffixes.sh | exclude_groups.sh | extract_acls.sh $extract_acl_opt | \
		ldapadd -D cn=root -w $ldap_pwd 2>&1 | $logcmd
else
	# if groups were bulkloaded, don't exclude them
	cat $before_ldif | exclude_suffixes.sh | extract_acls.sh $extract_acl_opt | \
		ldapadd -D cn=root -w $ldap_pwd 2>&1 | $logcmd

fi

# drop the new index on ldap_desc table
# I think it will be rendered useless anyway by the sysstat tunings on LDAP_DESC, but it
# is takes up space and update time, so can be removed.  It can be recreated in
# about 35 minutes on a 3M user directory.
# 10/17: leaving this index defined because it can improve subtree searches
### echo $0: Drop the LDAP_DESC_DEID index : `date` 2>&1 | tee -a $logfile
### su - ldapdb2 -c "db2 connect to ldapdb2;db2 drop index LDAP_DESC_DEID;db2 terminate" 2>&1 | $logcmd

# Do some tunings after a large update
echo $0: Do a reorgchk and sysstat_tune.sh : `date` 2>&1 | tee -a $logfile
su - ldapdb2 -c "db2 connect to ldapdb2;db2 reorgchk update statistics on table all;db2 terminate" 2>&1 | $logcmd
su - ldapdb2 -c "$PWD/sysstat_tune.sh" 2>&1 | $logcmd

# check the differences
if [ $use_db2ldif_at_end = 1 ];then

	echo $0: Run db2ldif to extract the new directory : `date` 2>&1 | tee -a $logfile
	db2ldif -j -o after.ldif 2>&1 | $logcmd

	ls -ltr $before_ldif after.ldif
	# Here is example output for a 10K user database.
	# -rw-r--r--   1 root     system   11290286 Jan 10 17:13 before.ldif
	# -rw-r--r--   1 root     system   11280887 Jan 10 19:41 after.ldif
	# The difference in size is typically due to ALCs on Access Manager user objects
	# that were originally created with pdadmin.
	# The fixacls.sh script no longer makes those ACLs necessary.

	grep dn: $before_ldif | wc
	grep dn: after.ldif | wc
fi
echo $0: End : `date` 2>&1 | tee -a $logfile

