#!/bin/ksh

# Script to invoke the LDAP bulkload utility with options that provide the
# best performance.  The primary input to this script is the standard input 
# device, which contains the LDIF of the data to be bulkloaded.  Other input
# comes from a command line option and variables within this script.  A good
# reading of this script is recommended.

# This script breaks up the input stream of LDIF into "increments", the size
# of which are specified by the "increment" variable within this script.  The
# default setting of this variable is 2000100 which is a little over 2 million
# LDAP objects.  The reason for breaking the LDIF into increments is to
# overcome disk and memory constraints of the system on which the bulkload
# is to be performed.  The increment also provides a workaround for a
# limitation in some versions of bulkload in which bulkload crashes when
# the DB2 load tables it produces cannot exceed 2 GB in size.  Each
# increment loaded into LDAP with a separate invocation of the bulkload
# utility.

# Input - Standard input device containing the LDIF data to be
# bulk loaded

# Restrictions:
# This script must be run under the context of the root user.  It
# requires write authority to the current directory for temporary
# files.

# Warning: If large groups are to loaded using this script, the increment
# variable will not be satisfactory in splitting up the load, since it
# counts DNs, not members of the group.  A large group is a single DN.
# Instead, large groups should be loaded as separate invocations of the
# script or by using the bulkload utility directly.

# The script can be directed to stop at the next invocation of
# bulkload by writing "0" to a control file by the name of
# "continue.bulkload".
#
# The script records the dn's for first and last objects loaded in
# any increment into a log file of the script activity.  This
# can be used to aid in restarting the script after the last successfully
# loaded LDAP object.
#
# Disk requirements vary depending upon the LDIF input and the number of
# objects added in each incremental load.
# Disk requirements:
# - Temporary LDIF file created by this script.
# - DB2 load tables created by bulkload
# The number of objects in an incremental load is controlled by the
# increment variable within this script, so the disk space requirements
# vary.  If Access Manager users are being loaded, the disk space
# requirement is about 1.4 KB per LDAP object loaded (controlled by the
# increment variable) or about 2.8 GB of disk space for the default
# increment variable of 2000100.

# Increment variable
# Note for AM users, set increment to 4 times the number of
# AM users desired per increment
increment=2000100 # a little over half a million AM users (2M LDAP objects)

# Do not turn this option on.  It still has performance problems
use_bulkload_for_load_phase=0

usage(){
	print ""
	print Usage:  "$0 <drop_indexes|with_indexes> [<db2 import file field delimiter>]"
	print ""
	exit 1
}

if [ "X$1" = "X" ];then
	usage
fi
if [ $1 != "drop_indexes" ] && [ $1 != "with_indexes" ];then
	usage
fi
index_option=$1

unset db2delim
if [ X$2 != X ];then
	db2delim="-s \"$2\""
fi

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

export logfile=inc_bulk.out
rm -f $logfile #remove log file

# File used for temporary LDIF data
export ldif_file=bulkload.ldif

# Get ldap version
ldapvers=`ldapsearch -e | grep "SDK Version:" | $AWK '{print $NF}'`

# Specify the ldapimport directory (where DB2 load files go)
# Warning: This directory will be deleted and recreated.
# It is not cleaned up at the end, so delete it manually.
# Note: this is an environment variable used by the
# bulkload utility in versions before 5.1.
# LDAP 4.1 workaround: this must be a fully qualified name
export LDAPIMPORT=$PWD/ldapimport

# LDAP 5.1+ attaches "ldapimport" to the end of the named directory,
# so the ldapimport part of the path is being removed here for
# passing on to 5.1 bulkload later.
export ldapimport51=`echo $LDAPIMPORT  | awk '{
	if (substr($0,length($0)-9) != "ldapimport"){
		print $0
	} else {
		print substr($0,1,length($0)-11)
	}
}'`

# Other environment variables used by the bulkload
# utility before 5.1
export SCHEMACHECK=NO
export ACLCHECK=NO
export REMOVETMP=NO

# clean up the any previous import directory
rm -fr $LDAPIMPORT #comment out if REMOVETMP=YES
mkdir $LDAPIMPORT

# kill slapd if running
if [ X`ps -ef | grep slapd | grep -v grep | $AWK '{print $2}'` != X ];then
	print "$0: stopping the LDAP server"
	kill -9 `ps -ef | grep slapd | grep -v grep | $AWK '{print $2}'`
fi
while [ X`ps -ef | grep slapd | grep -v grep | $AWK '{print $2}'` != X ];do
	sleep 1
done

echo 1 >continue.bulkload

rm -f $ldif_file


# Define the invoke_bulkload temporary helper script
# This script invokes the bulkload utility with the
# most efficient parameter settings.

cat << EOF >invoke_bulkload.sh
#!/bin/ksh
# This is a temporary helper script created automatically by
# $0.  It can be deleted, since it will be re-created
# on the next usage of $0.

# Function to check whether user has specified to terminate
# bulkload at the next checkpoint.
continue_check(){

	if [ "X\`cat continue.bulkload\`" != "X1" ];then

		# Cleanup status file from this bulkload
		rm -f /usr/ldap/etc/bulkload_status*      #for AIX
		rm -f /opt/IBMldaps/etc/bulkload_status*  #for Solaris

		rm continue.bulkload

		exit 1
	fi
}

continue_check

# Invoke the bulkload utility

echo start time bulkload parse: \`date\` 2>&1 | tee -a $logfile

# clean up the any previous import directory
rm -fr $LDAPIMPORT #comment out if REMOVETMP=YES
mkdir $LDAPIMPORT

# Note the LDR_CNTRL environment variable used below increases the process size limit
# to 2 GB on AIX systems.  It has no affect on other operating systems.

if [ $ldapvers -lt 410 ];then

	# For LDAP 3.2:
	ACTION=PARSEONLY LDR_CNTRL=MAXDATA=0x80000000 bulkload -i $ldif_file -a no 2>&1 | tee -a $logfile
	if [ $? != 0 ];then
		echo 0 >continue.bulkload
	fi

else if [ $ldapvers -lt 510 ];then

	# For LDAP 4.1
	# Note: the -L option does not work, but the LDAPIMPORT environment variable still
	# does.  Also, bulkload 4.1 has bugs that make parameter order important.

	LDR_CNTRL=MAXDATA=0x80000000 bulkload -i $ldif_file -S no -A no $db2delim -a parseonly 2>&1 | tee -a $logfile
	if [ $? != 0 ];then
		echo 0 >continue.bulkload
	fi

	# LDAP 4.1 work around
	# Initial versions of LDAP 4.1 do not create the SRC table, so create it here
	# just as the 3.2.2 bulkload would have.

	# check to see if bulkload 4.1 has been fixed
	if [ ! -f $LDAPIMPORT/aclsrc ];then

		print "Detected that the bulkload parse phase did not create the aclsrc table load file."  | tee -a $logfile
		print "Generating the missing load file and ldapdb2.ddl load statements now." | tee -a $logfile

		# Create the aclsrc table using the first and last
		# eids from the ldap_desc table
		$AWK -v first=\`head -1 $LDAPIMPORT/ldap_desc | cut -d "," -f1\` \
		    -v last=\`tail -1 $LDAPIMPORT/ldap_desc | cut -d "," -f1\` \
		'BEGIN{
			for (i=first;i<=last;i++){
				print i",-1,-1"
			}
		}' >$LDAPIMPORT/aclsrc

		# The following section updates the bulkload created ldapdb2.ddl
		# load script file to include the aclsrc table

		# initialize the temporary file
		rm -f $0.tmp

		cat $LDAPIMPORT/ldapdb2.ddl | $AWK -v tf=$0.tmp 'BEGIN{

			# find the start of the ldap_desc load statements
			while (getline > 0){
				print \$0
				if (index(\$0,"ldap_desc")){
					break
				}
			}

			# save the ldap_desc load statements for future use
			print \$0 >tf
			while (getline > 0){
				print \$0
				print \$0 >tf
				if (\$1 == ""){
					break
				}
			}

			# add aclsrc load statements to the output stream by
			# modifying the saved ldap_desc load statements
			sedcmds = " | sed 's/ldap_desc/aclsrc/g'"
			sedcmds = sedcmds " | sed 's/LDAP_DESC/SRC/g'"
			close(tf)
			system("cat "tf sedcmds)

			# print the remaining load statements
			while (getline > 0){
				print \$0
			}

		}' >$LDAPIMPORT/ldapdb2.ddl.new

		mv $LDAPIMPORT/ldapdb2.ddl $LDAPIMPORT/ldapdb2.ddl.orig
		mv $LDAPIMPORT/ldapdb2.ddl.new $LDAPIMPORT/ldapdb2.ddl
		chmod +x $LDAPIMPORT/ldapdb2.ddl

		# remove temporary file
		rm -f $0.tmp

	fi
	# end of LDAP 4.1 aclsrc table workaround

else # 5.1+

	if [ $use_bulkload_for_load_phase = 1 ];then
		# -b option is only in private version from development
		LDR_CNTRL=MAXDATA=0x80000000 bulkload -b -i $ldif_file -L $ldapimport51 -S no -A no -I no -C yes $db2delim -a parse_and_load 2>&1 | tee -a $logfile
	else
		LDR_CNTRL=MAXDATA=0x80000000 bulkload -i $ldif_file -L $ldapimport51 -S no -A no $db2delim -a parseonly 2>&1 | tee -a $logfile
	fi
	if [ $? != 0 ];then
		echo 0 >continue.bulkload
	fi

fi
fi # ldap version check

# this file grows with each parse
# for LDAP 4.1
rm -f /tmp/slapd.errors
rm -f /var/ldap/slapd.errors
# for LDAP 5.1+
rm -f /var/ldap/bulkload.log

continue_check

if [ $use_bulkload_for_load_phase != 1 ] || [ $ldapvers -lt 510 ] ;then

echo start time bulkload load: \`date\` 2>&1 | tee -a $logfile

# Someday, it is hoped that the bulkload loadonly option will be fast.  If that
# happens in some version of LDAP, the status file cannot be erased.
if [ $use_bulkload_for_load_phase != 1 ] || [ $ldapvers -lt 510 ] ;then

	# Erase the status file, since we will be invoking the bulkload generated load
	# scripts directly.

	if [ $index_option = "with_indexes" ];then

		# For loading without dropping indexes
		rm -f /usr/ldap/etc/bulkload_status* #for AIX
		rm -f /opt/IBMldaps/etc/bulkload_status* # for Solaris
	fi
fi

if [ $index_option = "drop_indexes" ];then
	# For load with dropping indexes

	# Note the LDR_CNTRL environment variable used below increases the
	# process size limit to 2 GB on AIX systems.  It has no affect on other
	# operating systems.

	if [ $ldapvers -lt 410 ];then

		# For LDAP 3.2:
		ACTION=LOADONLY LDR_CNTRL=MAXDATA=0x80000000 bulkload -i $ldif_file -a on 2>&1 | tee -a $logfile
		if [ $? != 0 ];then
			echo 0 >continue.bulkload
		fi

	else if [ $ldapvers -lt 510 ];then

		LDR_CNTRL=MAXDATA=0x80000000 bulkload -i $ldif_file -I yes -S no -a loadonly 2>&1 | tee -a $logfile
		if [ $? != 0 ];then
			echo 0 >continue.bulkload
		fi

	else
		# 5.1+

		LDR_CNTRL=MAXDATA=0x80000000 bulkload -i $ldif_file -L $ldapimport51 -S no -A no -I yes -a loadonly 2>&1 | tee -a $logfile
		if [ $? != 0 ];then
			echo 0 >continue.bulkload
		fi

	fi
	fi

else

	# Find the ldapdb2 instance owner from the load script
	ldapdb2_user=\`grep "db2 connect to" $LDAPIMPORT/ldapdb2.ddl | $AWK '{print \$6}'\`

	if [ $ldapvers -lt 510 ];then

		# For load with indexes
		# Do not use bulkload for the load phase.  bulkload load phase got slower in 4.1
		# and also creates the aclsrc table, which has been worked around in another part
		# of this script.
		# LDR_CNTRL=MAXDATA=0x80000000 bulkload -i $ldif_file -I no -C yes -S no -a loadonly 2>&1 | tee -a $logfile
		# if [ $? != 0 ];then
		# 	echo 0 >continue.bulkload
		# fi

		# Invoke the load script directly as below, instead.
		# This is the same for both 3.2 and 4.1.
		su - \$ldapdb2_user -c "cd $LDAPIMPORT;$LDAPIMPORT/ldapdb2.ddl 2>&1" | tee -a $logfile
		if [ X\`cat $LDAPIMPORT/db2load.RC\` != X0 ];then
			echo 0 >continue.bulkload
		fi
	else
		# 5.1+

		if [ $use_bulkload_for_load_phase = 1 ];then
			LDR_CNTRL=MAXDATA=0x80000000 bulkload -i $ldif_file -L $ldapimport51 -S no -A no -I no -C yes -a loadonly 2>&1 | tee -a $logfile
			if [ $? != 0 ];then
				echo 0 >continue.bulkload
			fi
		else
			su - \$ldapdb2_user -c "cd $LDAPIMPORT;$LDAPIMPORT/ldapdb2.ddl 2>&1" | tee -a $logfile
			if [ X\`cat $LDAPIMPORT/db2load.RC\` != X0 ];then
				echo 0 >continue.bulkload
			fi
		fi
	fi
fi

fi # for use_bulkload_for_load_phase

rm -f $ldif_file

continue_check

EOF

chmod +x invoke_bulkload.sh


# Main

# For each increment of ldap objects, create a temporary
# ldif file and invoke the bulkload utility.

echo start time create temporary ldif: `date` 2>&1 | tee -a $logfile

# Cleanup status file from any previous use of bulkload
rm -f /usr/ldap/etc/bulkload_status* #for AIX
rm -f /opt/IBMldaps/etc/bulkload_status* # for Solaris

$AWK -v ldif_file=$ldif_file \
	-v index_option=$index_option \
	-v increment=$increment '{

	# put the line in the temp ldif file
	print $0 > ldif_file

	# if this is a dn line
	if ($1 == "dn:"){

		dn_line = $0

		# save the first dn
		if (!not_first){
			not_first = 1
			system("echo first "dn_line" | tee -a $logfile")
		}

		# check for and save the last dn
		dn_count++
		if (dn_count >= increment){
			last_dn = 1
			system("echo last "dn_line" | tee -a $logfile")
		}
	}

	# if working on the last dn, find the null line that
	# terminates it and invoke bulkload
	if (last_dn && $0 == ""){
		not_first = 0
		last_dn = 0
		dn_count = 0

		close(ldif_file)

		system("invoke_bulkload.sh")

		system("echo start time create temporary ldif: `date` 2>&1 | tee -a $logfile")
	}
}END{

	# bulkload any left over dns
	if (not_first){
		system("echo last "dn_line" | tee -a $logfile")
		close(ldif_file)
		system("invoke_bulkload.sh")
	}
}'

# Cleanup status file from any previous use of bulkload
rm -f /usr/ldap/etc/bulkload_status* #for AIX
rm -f /opt/IBMldaps/etc/bulkload_status* # for Solaris

echo stop time: `date` 2>&1 | tee -a $logfile

rm -f invoke_bulkload.sh
rm -f continue.bulkload

rm -f $ldif_file
