#!/bin/ksh

# The primary purpose of this script is to break up an input
# stream of LDIF data containing one or more large group
# objects.  A large group object is defined as one having
# a large number of members.  The LDIF is broken up into
# the creation of the group object with no members followed
# by one or more modifications to add its members.  

# The reason for creating large groups in this way is to work
# around a couple of problems in LDAP:
# - For LDAP 3.2.1, creating a large group in one operation
#   results in an inefficient storage of that group in DB2,
#   resulting in performance problems when LDAP accesses that
#   group.
# - creating a large group requires a large amount of
#   storage in the LDAP directory holding DB2 tablespace1
#   (e.g. in ldapdb2 user's home directory).  This space
#   is used for the transaction log that is used to back
#   out the request in case it fails.  If this storage is
#   exceeded, the database is left corrupted and must
#   be restored from a previous back up.
#
# The secondary function of this script is to use ldapadd
# to load whatever LDIF is in the input stream.
#
# Input - Standard input device containing the LDIF data to be
# loaded
#
# Restrictions:
# This script can be run under the context of any user.  It
# requires write authority to the current directory for temporary
# files.

# Warning: Do not use this script to load a large number of
# LDAP objects.  The bulkload facility is much faster for
# that purpose.  Note this is different from a single object
# with a large number of members, which this script is
# designed to handle.

# This script divides large groups into increments that are
# separately loaded.  This keeps the space for the db2 transaction
# log to a managable size.
#
# The script can be directed to stop at the next invokation of
# of ldapadd or the  next "dn" by writing "0" to a control file
# by the name of "continue.groupload".
#
# The script records the names of first and last members added
# to a group in any increment.  This activity is logged to a file.
# The log file can aid in restarting the script at the last known
# point.
#
# Disk requirements vary depending upon the LDIF input and
# the number of members added per increment.
# Disk requirements:
# - tablespace in the LDAP instance owners home directory:  TBD.
#   expressed in approximate number of bytes per member in an
#   increment.  For example, for 10K members, the storage is
#   about TBD.
#   This directory also stores the temporary transaction log

# Increment variable
# This is the number of members to be added at one time.
increment=10000

# LDAP admin password
if [ X$ldap_pwd = X ];then
	ldap_pwd=fsaustin
fi

export logfile=inc_group.out
rm -f $logfile #remove log file

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

# Make sure slapd running for ldapadd
print "$0: Start LDAP"
slapd_pid=`ps -ef | grep slapd |
	$AWK '{if ($1 == "ldap")print $2}'`
if [[ "X$slapd_pid" = "X" ]] then
	# One of the following will start LDAP
	slapd
	# note that LDAP_MAXCARD=NO disables the persistent sysstat tunes in LDAP 5.2.
	LDAP_MAXCARD=NO ibmslapd
	sleep 5
	ldapsearch -s base -b "" "objectclass=*" >/dev/null 2>&1
	while [ $? != 0 ];do
		sleep 1
		ldapsearch -s base -b "" "objectclass=*" >/dev/null 2>&1
	done
fi

echo 1 >continue.groupload

# main 

$AWK -v increment=$increment \
     -v ldap_pwd=$ldap_pwd '

function ldapadd_pipe_write(cmd){

	if (debug == 0){
		print cmd | pipe_cmd
	} else {
		print cmd >"ldapadd_debug.log"
	}
}

function ldapadd_pipe_close(){

	if (debug == 0){
		close(pipe_cmd)
	} else {
		print "close" >"ldapadd_debug.log"
	}
}

function cleanup(){

	if (working_on_group) {
		system("echo last "last_member" | tee -a $logfile")
	}
	system("echo last "dn_line[0]" | tee -a $logfile")
	ldapadd_pipe_close()
	system("echo end time ldapadd: `date` 2>&1 | tee -a $logfile")
	system("rm -f continue.groupload")

}

function continue_check(){

	getline < "continue.groupload"
	if ($1 != 1){
		cleanup()
		exit
	}
	close ("continue.groupload")
}

function finish_and_more_member_check(){

	# Finish out the possibly partial ldif for this group
	# object.
	# If this is the first time called for this group,
	# it causes the group object to be created.
	# The group object contains a single member, namely
	# the first member in the ldif.
	# If this is the subsequent time called for this
	# group, it finishes out the incremental load
	# of members for that group.

	# If we just completed an incremental load of members,
	# close out the ldapadd and take a timing

	if (mem_count == increment) {

		ldapadd_pipe_write("")
		ldapadd_pipe_close()
		not_first = 0

		# print out a time stamp
		system("echo end time ldapadd: `date` 2>&1 | tee -a $logfile")
	}

	working_on_group = 0

	# If we are still in the middle of a dn definition
	# determine if the definition has ended.

	if ($1 != "") {

		# Check to see if there are any other members in
		# the group.  We have a group, but it could have had
		# only a single member or the last incremental
		# add could have finished out the entire group.

		getline
		line_save = $0
		if ( $1 != "") {

			attr = substr($1,1,length($1)-1)

			# Start the ldif to add the remaining member(s) to
			# the group.
			ldapadd_pipe_write("")
			for (i=0;i<dnln_cnt;i++){
				ldapadd_pipe_write(dn_line[i])
			}
			ldapadd_pipe_write("changetype: modify")
			ldapadd_pipe_write("add: "attr)

			if ( $1 == "member:") {

				last_member = line_save
				continue_check()

				system("echo first "line_save" | tee -a $logfile")

				working_on_group = 1
				mem_count = 1
			}
		}
		ldapadd_pipe_write(line_save)
	}
}


BEGIN{
	system("echo start time ldapadd: `date` 2>&1 | tee -a $logfile")
	pipe_cmd = "ldapadd -D cn=root -w "ldap_pwd" 2>&1 | tee -a $logfile"
	debug = 0
}
{
	# process a line of ldif

	save_line = $0
	if ($1 == "dn:"){
		continue_check()
	}
	$0 = save_line

	# send the line of ldif to ldapadd
	ldapadd_pipe_write(save_line)

	# if this is a dn line
	if ($1 == "dn:"){

		dnln_cnt = 0
		dn_line[dnln_cnt++] = $0
		notdone = 1
		while (notdone) {
			getline
			if (substr($0,1,1)==" "){
				ldapadd_pipe_write($0)
				dn_line[dnln_cnt++] = $0
			}else{
				notdone = 0
			}
		}

		# save the first dn
		if (!not_first){
			not_first = 1
			system("echo first "dn_line[0]" | tee -a $logfile")
		}

		# If this is not a changetype line, then add changetype.
		# Note: ldapadd does not seem to handle some dns with changetype
		# and others without

		if ($1 != "changetype:") {

			ldapadd_pipe_write("changetype: add")

		}

		# send the line of ldif to ldapadd
		ldapadd_pipe_write($0)

	} else if (working_on_group) {

		# working on a group

		# check for end of member definition
		if ($1 != "member:") {

			system("echo last "last_member" | tee -a $logfile")
			if ($1 == "") {
				finish_and_more_member_check()
			}
		} else {
			last_member = $0
			
			# increment the number of members in this
			# addition to the group
			mem_count ++

			# if we have reached the max number per
			# increment
			if ( mem_count == increment) {

				system("echo last "$0" | tee -a $logfile")

				# Check to see if there are any other members
				finish_and_more_member_check()
			}
		}

	} else {

		# This is ldif data, but we do not know if
		# it is for a group yet

		# member attribute means this is a group
		if ( $1 == "member:") {

			system("echo group "dn_line[0]" | tee -a $logfile")
			system("echo first "$0" | tee -a $logfile")
			system("echo last "$0" | tee -a $logfile")

			# Check to see if there are any other members
			finish_and_more_member_check()
		}
	}

}END{

	cleanup()

}'


