#!/bin/ksh

# Script to enable or disable DB2 system statistic tunes that improve the IDS
# directory server search performance, some for the general case and other
# specifically for TAM, but possibly applicable to other products.  Note that
# some of the tunes are also performed by the IDS directory server when
# it is started or the IDS runstats utility is executed.

# Restrictions:
# - This script must be run under the context of the root system user.
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It does not require write authority to the current directory.

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

usage(){
	print "Usage: $0 [enable|disable|check]"
	print "    where enable is the default"
	exit 1
}

if [ X$1 = X ]||[ X$1 = Xenable ];then
	tune_req=enable
elif [ X$1 = Xdisable ];then
	tune_req=disable
elif [ X$1 = Xcheck ];then
	tune_req=check
else
	usage
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import

ask_ldapdbinst

# args:  sysstat table name, IDS table name, stat column to modify
get_stat(){

	stat_tbl=$1
	tbl=$2
	istat=$3

	print "Obtaining the current setting"

if [ 0 = 1 ];then

	# Old way:

	if [ $stat_tbl = indexes ];then

		# For some reason, the sysstat.index table varies with user context,
		# so to get correct view, we just switch to the database manager user

		cat << EOFEOF
	cat << EOF | su - $ldapdbuser -c ksh
		. ~$ldapdbinst/sqllib/db2profile
		db2 connect to $ldapdbname >/dev/null
		db2 -x "select $istat from sysstat.$stat_tbl where tabname = '$tbl'"
		db2 terminate >/dev/null
EOF
EOFEOF
		cat << EOF | su - $ldapdbuser -c ksh |&
			. ~$ldapdbinst/sqllib/db2profile
			db2 connect to $ldapdbname >/dev/null
			db2 -x "select $istat from sysstat.$stat_tbl where tabname = '$tbl'"
			db2 terminate >/dev/null
EOF
		wait $!
	else
		print "db2 -x \"select $istat from sysstat.$stat_tbl where tabname = '$tbl'\""
		db2 -x "select $istat from sysstat.$stat_tbl where tabname = '$tbl'" |&
		wait $!
	fi
	read -p stat

	# End Old way
fi

	write_db2_ksh "db2 -x \"select max($istat) from sysstat.$stat_tbl where tabname = '$tbl'\""
	read_db2_ksh
	stat=$db2_ksh_result

	if [ "X$stat" = "X-" ];then
		unset stat
		print "\tTable does not exist at this time."
	else
		print "\t$stat"
	fi
}

check_and_enable(){
	tabname=$1
	ids_tune_flag=$2
	desired_value=$3

	print ""
	if [ $ids_tune_flag = 1 ];then
		print "Checking the the $tabname system statistic tune.  Note this tune is also enabled when"
		print "the IBM Directory Server is started and when the IDS runstats utility is run."
	else
		print "Checking the the $tabname system statistic tune."
	fi

	get_stat tables $tabname card

	# if table exists
	if [ "X$stat" != X ];then
		attempt_count=$(( attempt_count + 1 ))

		if [ $stat = $desired_value ];then
			print "The tune is already enabled.  No action necessary."
			already_enabled_count=$(( already_enabled_count + 1 ))
		else
			print "Performing the update to enable the tune"
			write_db2_ksh "db2 \"update sysstat.tables set card = $desired_value where tabname = '$tabname'\""
			read_db2_ksh
		fi
	fi
}

disable_sysstat_tune(){

	tbl=$1
	tuned_value=$2

	print ""
	print "Processing the $tbl table"
	print ""

	get_stat tables $tbl card

	if [ "X$stat" != X ];then
		attempt_count=$(( attempt_count + 1 ))

		if [ $stat != $tuned_value ];then

			already_disabled_count=$(( already_disabled_count + 1 ))
			print "The DB2 system statistic tune is already disabled"

		else
			# Old way, too slow:
			# print "The system statistic tuning for $tbl is being disabled"
			# print "  Note that this could take several minutes."
			#
			# newcard=`db2 connect to $ldapdbname >/dev/null;
			# 	db2 select count\(*\) from $tbl |
			# 	$AWK '{if (index($1,"-----")){getline;print $1}}'`
			# cmd="db2 update sysstat.tables set card = $newcard where tabname = '$tbl'"

			print "Obtaining the original cardinality value"
			get_stat columns $tbl colcard
			new_stat=$stat

			get_stat indexes $tbl first2keycard
			if [ $stat -gt $new_stat ];then
				new_stat=$stat
			fi

			print "Performing the update to disable the tune"
			write_db2_ksh "db2 \"update sysstat.tables set card = $new_stat where tabname = '$tbl'\""
			read_db2_ksh verbose
		fi
	fi
}


# Main program

# Commands will be piped to and from this process
start_db2_ksh

if [ 0 = 1 ];then
	print ""
	# Connect to the database
	print "db2 connect to $ldapdbname >/dev/null"
	db2 connect to $ldapdbname >/dev/null
fi

if [ $tune_req = check ];then

	print ""
	print "Getting the current db2 system statistics tune settings"
	print ""

	get_stat tables LDAP_DESC card
	get_stat tables LDAP_ENTRY card
	get_stat tables SECAUTHORITY card
	get_stat tables CN card
	get_stat tables REPLCHANGE card

	# if REPLCHANGE table exists
	if [ "X$stat" != X ];then

		# cannot use get_stat, because of colname = 'ID' additional clause

		write_db2_ksh "db2 -x \"select colcard, high2key from sysstat.columns where colname = 'ID' and tabname = 'REPLCHANGE'\""

		read_db2_ksh
		print "\t$db2_ksh_result"
	fi

elif [ $tune_req = enable ];then

	# enable the tunes

	already_enabled_count=0
	attempt_count=0

	print ""
	print "Enabling the DB2 system statistic tunes.  These tunes make the DB2 optimizer"
	print "think the affected tables are very large.  LDAP searches that combine multiple"
	print "attributes with the affected tables will result in affected tables being the"
	print "last criterion checked in determining which LDAP entries satisfy the search"
	print "parameters."

	# check_and_enable parms are tabname, ids_tune_flag, desired_value

	check_and_enable LDAP_DESC 1 9000000000000000000
	check_and_enable LDAP_ENTRY 0 9000000000000000000

	# Note that these tables use 9E10, instead of 9E18.  This should make DB2 pick
	# them before the LDAP_DESC and LDAP_ENTRY tables, but after most other tables.

	check_and_enable SECAUTHORITY 0 90000000000
	check_and_enable CN 0 90000000000

	check_and_enable REPLCHANGE 1 9000000000000000000

	# if REPLCHANGE table exists
	if [ "X$stat" != X ];then

		write_db2_ksh "db2 -x \"select colcard, high2key from sysstat.columns where colname = 'ID' and tabname = 'REPLCHANGE'\""
		read_db2_ksh
		stat=$db2_ksh_result
		print "\t$stat"

		if [ "X$stat" = "X9000000000000000000 2147483646" ];then
			print "The tune is already enabled.  No action necessary."
		else
			write_db2_ksh "db2 \"update sysstat.columns set colcard=9E18, high2key='2147483646' where colname = 'ID' and tabname = 'REPLCHANGE'\""
			read_db2_ksh
		fi
	fi
	print ""
	if [ $already_enabled_count = 0 ];then
		print "DB2 system statistic tunes have successfully been enabled"
	elif [ $already_enabled_count = $attempt_count ];then
		print "DB2 system statistic tunes were already enabled.  No action was taken."
	else
		print "One or more DB2 system statistic tunes were already enabled"
		print "DB2 system statistic tunes have successfully been enabled"
	fi
	print ""

else
	# disable the tunes

	already_disabled_count=0
	attempt_count=0

	print ""
	print "Disabling the DB2 system statistic tunes"

	disable_sysstat_tune LDAP_DESC 9000000000000000000
	disable_sysstat_tune LDAP_ENTRY 9000000000000000000

	disable_sysstat_tune SECAUTHORITY 90000000000
	disable_sysstat_tune CN 90000000000

	print ""
	if [ $already_disabled_count = 0 ];then
		print "DB2 system statistic tunes have successfully been disabled"
	elif [ $already_disabled_count = $attempt_count ];then
		print "DB2 system statistic tunes were already disabled.  No action was taken."
	else
		print "One or more DB2 system statistic tunes were already disabled"
		print "DB2 system statistic tunes have successfully been disabled"
	fi
	print ""
fi

stop_db2_ksh

