#!/bin/ksh

# Restrictions:
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It requires write authority to the current directory.

# Use the following guidelines to determine whether the IBM Directory
# server is performing well.  By default, this script should run in 
# less than a 2 seconds even with randomly chosen users.
#
# The first time this script is run, it may take more than a second.
# This can happen if the DB2 buffer pool (cache) has not been
# loaded.
#
# If performance is not within these paremeters, review the Performance
# Tuning Guide.

action=invoke_and_process_output
test_user=$1
test_user_password=$2
domain=$3

if [ X$2 = X ];then 
	print "Usage: $0 <user_principalname> <user_password> [<domain>]"
	exit 1
fi

if [ $1 = "-run_with_timing" ];then
	action=run_with_timing
	test_user=$2
	test_user_password=$3
	domain=$4
fi

if [ X$domain = X ];then
	domain=Default
fi

# import the am_tune_ldap.sh utilities

if [ `uname` = "SunOS" ];then
        AWK=nawk
else
        AWK=awk
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import

ask_ldap_cred   # nice side effect: makes sure LDAP is started

# test directory server performance

# determine the user data format: standard or minimal

if [ X"$dataformat" = X ];then

	echo ldapsearch $ldap_cred -s base -b secauthority=default \"objectclass=*\" version
	amvers=`ldapsearch $ldap_cred -s base -b secauthority=default "objectclass=*" version | grep version`

	if [ X"$amvers" = X ];then

		ldapsearch $ldap_cred -s base -b secauthority=default "objectclass=*" version
		print $0: failed
		exit 1
	fi
	print "\t$amvers"
	if [ "$amvers" = "version=6.0" ];then
		dataformat=minimal
	else
		dataformat=standard
	fi
	export dataformat
fi

test_registry_perf(){

	print "Find the test user"

	getsufcmdl="ldapsearch -L $ldap_cred -s base -b \"cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration\" \"objectclass=*\" ibm-slapdSuffix"

	# For each suffix object, attempt to find the user

	# Use a background pipe
	eval $getsufcmdl 2>/dev/null | grep -i ibm-slapdSuffix |
		$AWK -F":" '{print $2}' |&

	# read from background pipe
	read -p suf
	found=0
	while [ $found = 0 ]&&[ "X$suf" != X ];do

		ldapsearch $ldap_cred -s base -b "$suf" "objectclass=*" dn >/dev/null 2>&1

		# if ldap entry exists
		if [ $? != 32 ];then

			# search the suffix
			echo ldapsearch $ldap_cred -s sub -b \"$suf\" \"secDomainId=$domain%$test_user\"
			dn=`ldapsearch $ldap_cred -s sub -b "$suf" "secDomainId=$domain%$test_user" dn 2>/dev/null`

			if [ $? = 0 ];then
				if [ X"$dn" != X ];then
					print "\t"$dn
					found=1
					found_suf=$suf
				fi
			else
				ldapsearch -L $ldap_cred -s sub -b "$suf" "secDomainId=$domain%$test_user"
				print ""
				print $0: failed
				exit 1
			fi
		fi

		# read from background pipe
		read -p suf
	done
	wait

	if [ $found = 0 ];then
		print $0: failed - User not found
		exit 1
	fi

	print "Test the user's password"

	if [ $dataformat = minimal ];then

		userdn=`ldapsearch $ldap_cred -s sub -b "$found_suf" "secDomainId=$domain%$test_user" secDN |
			grep -i secDN | $AWK -F"=" '{print substr($0,index($0,$2))}'`
	else
		userdn=`ldapsearch $ldap_cred -s sub -b "$found_suf" "secDomainId=$domain%$test_user" dn |
			$AWK -F"," '{print substr($0,index($0,$2))}'`
	fi

	echo ldapsearch -p $ldap_port -D \"$userdn\" -w \"$test_user_password\" -s base -b \"$userdn\" \"objectclass=*\" dn
	ldapsearch -p $ldap_port -D "$userdn" -w "$test_user_password" -s base -b "$userdn" "objectclass=*" dn >/dev/null

	if [ $? != 0 ];then
		print $0: failed
		exit 1
	fi

	print "Determine the user's group membership(s)"

	userdnsuf=`echo $userdn | $AWK -F"," '{print $NF}'`

	echo ldapsearch $ldap_cred -s base -b \"$userdn\" \"objectclass=*\" ibm-allgroups
	ldapsearch $ldap_cred -s base -b "$userdn" "objectclass=*" ibm-allgroups | grep -i ibm-allgroups

	if [ $? != 0 ];then
		print $0: failed
		exit 1
	fi
}

if [ $action = run_with_timing ];then
	time test_registry_perf
else
	print "Performing the test to be timed"
	print ""

	test_registry_perf.sh -run_with_timing $test_user $test_user_password $domain 2>&1 | $AWK -v pn=$0 '{
		print $0
		if ($1 == "real"){
			time = $2
		}
	}END{
		if ( time ) {
			print ""
			time = substr(time,index(time,"m")+1)
			time = substr(time,1,index(time,"s")-1)
			if ( time > 2 ){
				print pn" : failed - test completed in more than 2 seconds"
				print "This could be due to DB2 cache warm up time.  Pick another random user and"
				print "re-run the test.  If the performance test continues to fail, verify the"
				print "directory server has been tuned.  If that fails to solve the problem, investigate"
				print "the problem by measuring individual searches and performing a DB2 statement"
				print "monitor on the slow search."
			}else{
				print pn" : succeeded - test completed in less than 2 seconds"
			}
		}
	}'
fi
